import sys
import os
from PyQt5.QtWidgets import*
from PyQt5.QtCore import *
from PyQt5.Qt import QStandardItemModel, QStandardItem
from PyQt5.QtGui import *
from GraphKey.app.modules.data_export.google_drive_interface.google_drive_export_screen import *
from GraphKey.app.modules.data_export.local_export.local_machine_select_file import *

class ExportScreen(QWidget):
    """
    A QWidget class defining the Export Screen. This screen simply gives the user the option to either export files to their local machine or a Google Drive Account.
    """
    def __init__(self, app, list_of_file_paths, graph_page):
        """Initializes the Export Screen.
           :param: app - Variable pointing to the parent application, used for processing events.
           :param: list_of_file_paths - List of files user can choose to export from.
           :param: graph_page - Variable linking to the graph_creation_page instance.
        """
        super().__init__()
        self.widget_app = app
        self.file_list = list_of_file_paths
        self.graph_creation_page = graph_page
        self.make_window()

    def make_window(self):
        """Creates the Export window and all of its elements."""
        layout = QVBoxLayout()
        drive_layout = QVBoxLayout()


        #BUTTON AREA
        button_widget = QWidget()
        button_layout = QHBoxLayout()
        cancel = QPushButton("Cancel")
        cancel.clicked.connect(lambda: self.cancel())
        drive = QPushButton("Export to Google Drive")
        drive.clicked.connect(lambda: self.drive())
        local = QPushButton("Export to Local Machine")
        local.clicked.connect(lambda: self.local())

        button_layout.addWidget(local)
        button_layout.addWidget(drive)
        button_layout.addWidget(cancel)
        button_widget.setLayout(button_layout)

        #WHOLE WIDGET
        drive_layout.addWidget(button_widget)
        drive_widget = QWidget()
        drive_widget.setLayout(drive_layout)

        layout.addWidget(drive_widget)
        self.setLayout(layout)
        self.setWindowTitle("Select Export Location")
        self.show()

    def local(self):
        """If the user chooses to export to their local machine, this function will show them the Local Machine Select File window."""
        export = LocalMachineSelectFile(self.widget_app, self.file_list, self.graph_creation_page)
        export.show() 
        self.close()

    def drive(self):
        """If the user chooses to export to a Google Drive account, this function will show them the Google Drive Export Screen."""
        export = GoogleDriveExportScreen(self.widget_app, self.file_list, self.graph_creation_page)
        export.show()
        self.close()

    def cancel(self):
        """If the user chooses to cancel, this function will close the window."""
        self.close()

