from threading import Thread
import functools
import sys
import signal
from contextlib import contextmanager
from pydrive.drive import GoogleDrive
from GraphKey.app.modules import path_globals
from pathlib import Path
from pydrive.auth import GoogleAuth
GoogleAuth.DEFAULT_SETTINGS['client_config_file'] = Path(path_globals.settings_folder_path / "client_secrets.json")

def upload_file(drive, filename, current_folder_id, title):
    """This function uploads a file to a Google Drive account.
       :param: drive - The link to the Google Drive account.
       :param: filename - The file to be uploaded.
       :param: current_folder_id - The folder the file should be uploaded to.
       :param: title - The title of the file to be uploaded.
    """
    if (current_folder_id != 0):
        file1 = drive.CreateFile({'title': title, 'parents': [{'kind': 'drive#fileLink', 'id': current_folder_id}]})
    else:
        file1 = drive.CreateFile({'title': title})
    file1.SetContentFile(filename)
    file1.Upload()

def create_folder(drive, foldername, current_folder_id):
    """This function creates a file in a Google Drive account.
       :param: drive - The link to the Google Drive account.
       :param: foldername - The folder to create.
       :param: current_folder_id - The folder the new folder should be created in.
    """
    if (current_folder_id != 0):
        folder = drive.CreateFile({'parents': [{'id': current_folder_id}],'title' : foldername, 'mimeType' : 'application/vnd.google-apps.folder'})
    else:
        folder = drive.CreateFile({'title' : foldername, 'mimeType' : 'application/vnd.google-apps.folder'})
    folder.Upload()

def list_folders_files(drive, current_folder_id):
    """This function retrieves the contents of a folder in a Google Drive Account.
       :param: drive - The link to the Google Drive account.
       :param: current_folder_id - The folder to retrieves contents from.
       :return: Returns a list of files.
    """
    if (current_folder_id != 0):
        fileList = drive.ListFile({'q': "'" + current_folder_id + "'" + " in parents and trashed=false"}).GetList()
    else:
        fileList = drive.ListFile({'q':"'root' in parents and trashed=false"}).GetList()
    return fileList

def timeout(seconds_before_timeout):
    """This function enables a timeout and raises an exception on timeout.
       :param: seconds_before_timeout - The length of the timeout.
    """
    def deco(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            res = [Exception('timeout')]
            def newFunc():
                try:
                    res[0] = func(*args, **kwargs)
                except Exception as e:
                    res[0] = e
            t = Thread(target=newFunc)
            t.daemon = True
            try:
                t.start()
                t.join(seconds_before_timeout)
            except Exception as e:
                raise e 
            ret = res[0]
            if (isinstance(ret, BaseException)):
                raise ret 
            return ret
        return wrapper
    return deco

@timeout(30)
def init():
    """This function handles Google Drive authentication and authentication exceptions, as well as enables a 30 second timeout.
       :return: Returns a link to the Google Drive account.
    """
    try:
        gauth = GoogleAuth()
        timeout(seconds_before_timeout=30)(gauth.LocalWebserverAuth())
    except:
        print("Authentication Failed")
        return "AuthError"
    return GoogleDrive(gauth)

def init_drive():
    """This function starts the Google Drive authentication process and handles timeout exceptions.
       :return: Returns a link to the Google Drive account.
    """
    try:
        gd = init()
    except:
        print("Timeout")
        return "TimeoutError"
    return gd
