import sys
import os
import pickle
from PyQt5.QtWidgets import*
from PyQt5.QtCore import *
from PyQt5.Qt import QStandardItemModel, QStandardItem
from PyQt5.QtGui import *
import GraphKey.app.modules.data_export.google_drive_interface.google_drive_functions as gd
from GraphKey.app.modules.data_export.google_drive_interface.google_drive_select_account import *
from GraphKey.app.modules.data_export.google_drive_interface.google_drive_select_folder import *

class GoogleDriveSelectFile(QWidget):
    """
    A QWidget class defining the Google Drive Select File Screen. This screen shows the user the files they can select from to export to Google Drive.
    """

    def __init__(self, app, list_of_files, graph_page):
        """Initializes the Google Drive Select File Screen.
           :param: app - Variable pointing to the parent application, used for processing events.
           :param: list_of_file_paths - List of files user can choose to export from.
           :param: graph_page - Variable linking to the graph_creation_page instance.
        """
        super().__init__()
        self.widget_app = app
        self.graph_creation_page = graph_page
        configs = pickle.load(open(Path(path_globals.settings_folder_path / "google_drive_configs.p"), "rb"))
        self.current_account = configs["current_account"]
        self.file_list = list_of_files
        self.make_window()

    def make_window(self):
        """Creates the Select File window and all of its elements."""
        
        layout = QVBoxLayout()
        list_layout = QVBoxLayout()

        #BUTTON AREA
        button_widget = QWidget()
        button_layout = QHBoxLayout()
        cancel = QPushButton("Cancel")
        cancel.clicked.connect(lambda: self.cancel())
        export = QPushButton("Export Files")
        export.clicked.connect(lambda: self.export())

        button_layout.addWidget(export)
        button_layout.addWidget(cancel)
        button_widget.setLayout(button_layout)

        #SELECTED FILES AREA
        scroll_widget = QWidget()
        scroll_layout = QVBoxLayout()
        self.check_all = QCheckBox("Select All")
        self.check_all.stateChanged.connect(lambda:self.checked_all())
        scroll_layout.addWidget(self.check_all)
        self.check_box_list = []
        i = 0
        for f in self.file_list:
            self.check_box_list.append(QCheckBox(os.path.splitext(os.path.basename(self.file_list[i]))[0]))
            scroll_layout.addWidget(self.check_box_list[i])
            i = i+1
        scroll_widget.setLayout(scroll_layout)

        scroll_theme = QScrollArea()
        scroll_theme.setWidget(scroll_widget)
        scroll_theme.setWidgetResizable(True)

        #WHOLE WIDGET
        list_layout.addWidget(scroll_theme)
        list_layout.addWidget(button_widget)
        list_widget = QWidget()
        list_widget.setLayout(list_layout)

        layout.addWidget(list_widget)
        self.setLayout(layout)
        self.setWindowTitle("Select Files to Export to Google Drive")
        self.show()

    def export(self):
        """Once the user chooses to export, this function will create the Google Drive Select Folder screen."""
        checked_list = []
        if self.check_all.isChecked():
            checked_list = self.file_list
        else:
            i = 0
            for box in self.check_box_list:
                if box.isChecked():
                    checked_list.append(self.file_list[i])
                i = i + 1
        tree = FileTree(self.widget_app, self.current_account["account"], checked_list, self.graph_creation_page)
        tree.show()
        self.close()

    def checked_all(self):
        """If the user chooses to select all files, this function will check all the files in the list."""
        if self.check_all.isChecked():
            for box in self.check_box_list:
                box.setChecked(True)
            self.check_all.setChecked(False)
       
    def cancel(self):
        """If the user chooses to cancel, this function will close the window."""
        self.close()

