import sys
import os
import pickle
import PIL as pil
from PIL import Image
from PyQt5.QtWidgets import*
from PyQt5.QtCore import *
from PyQt5.Qt import QStandardItemModel, QStandardItem
from PyQt5.QtGui import *
from GraphKey.app.modules.data_export.google_drive_interface.google_drive_select_account import *
from GraphKey.app.modules.data_export.google_drive_interface.google_drive_select_folder import *
from GraphKey.app.modules.gui.graph_creation_window import *

class LocalMachineSelectFile(QWidget):
    """
    A QWidget class defining the Local Machine Select File Screen. This screen shows the user the files they can select from to export to the local machine.
    """

    def __init__(self, app, list_of_files, graph_page):
        """Initializes the Local Machine Select File Screen.
           :param: app - Variable pointing to the parent application, used for processing events.
           :param: list_of_file_paths - List of files user can choose to export from.
           :param: graph_page - Variable linking to the graph_creation_page instance.
        """
        super().__init__()
        self.widget_app = app
        self.graph_creation_page = graph_page
        self.file_list = list_of_files
        self.make_window()

    def make_window(self):
        """Creates the Select File window and all of its elements."""
        
        layout = QVBoxLayout()
        list_layout = QVBoxLayout()

        #BUTTON AREA
        button_widget = QWidget()
        button_layout = QHBoxLayout()
        cancel = QPushButton("Cancel")
        cancel.clicked.connect(lambda: self.cancel())
        export = QPushButton("Export Files")
        export.clicked.connect(lambda: self.export())

        button_layout.addWidget(export)
        button_layout.addWidget(cancel)
        button_widget.setLayout(button_layout)

        #SELECTED FILES AREA
        scroll_widget = QWidget()
        scroll_layout = QVBoxLayout()
        self.check_all = QCheckBox("Select All")
        self.check_all.stateChanged.connect(lambda:self.checked_all())
        scroll_layout.addWidget(self.check_all)
        self.check_box_list = []
        i = 0
        for f in self.file_list:
            self.check_box_list.append(QCheckBox(os.path.splitext(os.path.basename(self.file_list[i]))[0]))
            scroll_layout.addWidget(self.check_box_list[i])
            i = i+1
        scroll_widget.setLayout(scroll_layout)

        scroll_theme = QScrollArea()
        scroll_theme.setWidget(scroll_widget)
        scroll_theme.setWidgetResizable(True)

        #WHOLE WIDGET
        list_layout.addWidget(scroll_theme)
        list_layout.addWidget(button_widget)
        list_widget = QWidget()
        list_widget.setLayout(list_layout)

        layout.addWidget(list_widget)
        self.setLayout(layout)
        self.setWindowTitle("Select Files to Export to Local Machine")
        self.show()

    def export(self):
        """Once the user chooses to export, this function will upload the files to the local machine to the chosen folder."""
        checked_list = []
        if self.check_all.isChecked():
            checked_list = self.file_list
        else:
            i = 0
            for box in self.check_box_list:
                if box.isChecked():
                    checked_list.append(self.file_list[i])
                i = i + 1
        folder = str(QFileDialog.getExistingDirectory(self, "Select Directory to Save File(s)"))
        if folder:
            msg = QMessageBox()
            msg.setWindowTitle("Saving")
            msg.setText("Please wait while your files are saved. Do not close this window.")
            msg.setModal(True)
            msg.show()
            self.widget_app.processEvents()
 
            for path in checked_list:
                figure, figure_dict = self.graph_creation_page.single_graph_generation_for_export(path)
                if figure is None:
                    return
                path_file = folder + "/" + os.path.splitext(os.path.basename(path))[0] + ".png"
                figure.write_image(file = path_file, format = 'png', scale = figure_dict["scale_factor"], width = figure_dict["width"], height = figure_dict["height"], validate = True)
                fig_image = pil.Image.open(path_file)
                fig_image.save(path_file, dpi=(figure_dict["dpi"],figure_dict["dpi"]))
            msg.close()
            self.close()
    
    def checked_all(self):
        """If the user chooses to select all files, this function will check all the files in the list."""
        if self.check_all.isChecked():
            for box in self.check_box_list:
                box.setChecked(True)
            self.check_all.setChecked(False)
    
    def cancel(self):
        """If the user chooses to cancel, this function will close the window."""
        self.close()
