import logging
import os
import subprocess

from GraphKey.app.modules.data_import.import_file import DataImport
import psutil


class DataEdit:
    """
    This class is designed to open the file within Excel straight from the application.
    """
    def __init__(self, file_path: str):
        """
        Initializes the class.
        :param file_path: The path to the file to edit
        """
        file_exists = os.path.isfile(file_path)
        if not file_exists:
            raise FileNotFoundError(f'{file_path} is not a valid file!')
        else:
            self._file_in_use = file_path

    def start_data_editing(self):
        """
        Launches Excel to edit the data file.
        """
        self.file = subprocess.Popen(['CALL', self._file_in_use], shell=True)

    def stop_data_editing(self):
        """
        Attempts to close the open Excel file and load the newly changed data back into the app.
        :return: The newly imported data.
        """
        try:
            logging.debug(f'Killing edit of {self._file_in_use}')
            psutil.Process(self.file.pid).children[0].kill()
        except:
            logging.info(f'File has already been closed!')

        return DataImport().import_data(self._file_in_use)
