import pickle

"""
File contains functions that write the information about a figure to a pickle file. The information can then be read back at a later time and be used to re-create the figure.
"""
def createScatterGraphPickle(filename, sheet_selected, figure_dict):
    """Writes information about a scatter graph to a pickle file
       :param: filename - File path of the pickle file to be written to.
       :param: sheet_selected - The name of the workbook sheet that was selected when this graph was created.
       :param: figure_dict - Dict containing configuration information about this graph.
    """
    outfile = open(filename, 'wb')
    createDictEntriesForUnspecifiedOptions(figure_dict)
    figure_dict["graph_type"] = "scatter"
    figure_dict["sheet_selected"] = sheet_selected

    pickle.dump(figure_dict, outfile)
    outfile.close()

def createBarGraphPickle(filename, sheet_selected, figure_dict):
    """Writes information about a bar graph to a pickle file
       :param: filename - File path of the pickle file to be written to.
       :param: sheet_selected - The name of the workbook sheet that was selected when this graph was created.
       :param: figure_dict - Dict containing configuration information about this graph.
    """
    outfile = open(filename, 'wb')
    createDictEntriesForUnspecifiedOptions(figure_dict)
    figure_dict["graph_type"] = "bar"
    figure_dict["sheet_selected"] = sheet_selected

    pickle.dump(figure_dict, outfile)
    outfile.close()

def createBoxGraphPickle(filename, sheet_selected, figure_dict):
    """Writes information about a box graph to a pickle file
       :param: filename - File path of the pickle file to be written to.
       :param: sheet_selected - The name of the workbook sheet that was selected when this graph was created.
       :param: figure_dict - Dict containing configuration information about this graph.
    """
    outfile = open(filename, 'wb')
    createDictEntriesForUnspecifiedOptions(figure_dict)   
    figure_dict["graph_type"] = "box"
    figure_dict["sheet_selected"] = sheet_selected

    pickle.dump(figure_dict, outfile)
    outfile.close()

def createDictEntriesForUnspecifiedOptions(figure_dict):
    """
    Populates the figure dictonary with default values for unspecified options
    """
    if figure_dict.get("labels") == None:
        figure_dict["labels"] = dict()
    if figure_dict.get("log_x") == None:
        figure_dict["log_x"] = False
    if figure_dict.get("log_y") == None:
        figure_dict["log_y"] = False

def createTemplatePickle(filename, template_dict):
    """Creates a template pickle file with information specified in template_dict.
       :param: filename - The file path for the pickle file to be written to.
       :param: template_dict - Dict containing template information.
    """
    outfile = open(filename, 'wb')
    pickle.dump(template_dict, outfile)
    outfile.close()


def getDataFromPickle(filename):
    """Fetches the data contained in a pickle file at file path filename.
       :param: filename - The path to the pickle file to be read.
       :return: Dictionary containing the data stored in the pickle file.
    """
    infile = open(filename, 'rb')
    data = pickle.load(infile)
    infile.close()
    return data
