from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
import pickle
from GraphKey.app.modules import path_globals
from pathlib import Path
from PyQt5.QtGui import QIntValidator


class EditGraphFrameLite(QScrollArea):
    """QScrollarea that contains other Graph Preference options."""
 
    figure_elements = [" ", "Title","X-Axis Title", "Y-Axis Title", "X-Axis Labels", "Y-Axis Labels", "Legend Title", "Legend Elements"]
    font_families = ["Arial", "Calibri", "Times New Roman", "Verdana"]
   
    def __init__(self, temp_dict):
        """Adds all the window elements."""
        super().__init__() 
        #create widget and layout that will hold graph editing options
        self.widget = QWidget()
        self.vbox = QVBoxLayout()
        self.widget.setLayout(self.vbox)
        self.setWidgetResizable(True)
        self.setWidget(self.widget)
        self.vbox.setAlignment(Qt.AlignTop)
        #SIZE LABEL AND ENTRY BOXES
        self.vbox.addWidget(QLabel("Size in Inches"))
        size_elements_frame = QFrame()
        size_elements_frame_layout = QHBoxLayout()
        size_elements_frame.setLayout(size_elements_frame_layout)
        self.width_entry_box = QLineEdit()
        self.width_entry_box.setText(str(temp_dict["width_inches"]))
        self.height_entry_box = QLineEdit()
        self.height_entry_box.setText(str(temp_dict["height_inches"]))
        size_elements_frame_layout.addWidget(self.width_entry_box)
        size_elements_frame_layout.addWidget(QLabel("by"))
        size_elements_frame_layout.addWidget(self.height_entry_box)
        self.vbox.addWidget(size_elements_frame)
        #ENABLE LOG X-AXIS CHECKBOX
        self.enable_log_x_axis_checkbox = QCheckBox("Enable logarithmic x-axis")
        self.enable_log_x_axis_checkbox.setChecked(temp_dict["log_x"])
        self.vbox.addWidget(self.enable_log_x_axis_checkbox)
        #ENABLE LOG Y-AXIS CHECKBOX
        self.enable_log_y_axis_checkbox = QCheckBox("Enable logarithmic y-axis")
        self.enable_log_y_axis_checkbox.setChecked(temp_dict["log_y"])
        self.vbox.addWidget(self.enable_log_y_axis_checkbox)
        #ENABLE ERROR BARS BAR GRAPH OPTION ONLY
        self.enable_error_bars_checkbox = QCheckBox("Enable error bars")
        self.enable_error_bars_checkbox.setChecked(temp_dict["enable_error_bars"])
        self.vbox.addWidget(self.enable_error_bars_checkbox)
        #FONTS FRAME
        fonts_frame = QFrame()
        fonts_frame_layout = QHBoxLayout()
        fonts_frame_layout.setContentsMargins(0,0,0,0)
        fonts_frame.setLayout(fonts_frame_layout)
        self.vbox.addWidget(fonts_frame)
        #FOR EACH TEXT
        #FONT SIZE SELECT
        title_font_size_frame = QFrame()
        title_font_size_frame_layout = QVBoxLayout()
        title_font_size_frame_layout.setContentsMargins(0,0,0,0)
        title_font_size_frame.setLayout(title_font_size_frame_layout)
        fonts_frame_layout.addWidget(title_font_size_frame)
        title_font_size_frame_layout.addWidget(QLabel("Title Font Size"))
        self.title_font_size_entry_box = QLineEdit()
        self.title_font_size_entry_box.setValidator(QIntValidator(0, 100))
        self.title_font_size_entry_box.setText(str(float(temp_dict["Title font size"])*float(3/4)))
        title_font_size_frame_layout.addWidget(self.title_font_size_entry_box)
        #FONT FAMILY SELECT
        title_font_family_frame = QFrame()
        title_font_family_frame_layout = QVBoxLayout()
        title_font_family_frame_layout.setContentsMargins(0,0,0,0)
        title_font_family_frame.setLayout(title_font_family_frame_layout)
        fonts_frame_layout.addWidget(title_font_family_frame)
        title_font_family_frame_layout.addWidget(QLabel("Title Font Family"))
        self.title_font_family_dropdown = QComboBox()
        self.title_font_family_dropdown.addItems(self.font_families)
        index = self.title_font_family_dropdown.findText(temp_dict["Title font family"])
        if index >= 0:
            self.title_font_family_dropdown.setCurrentIndex(index)
        title_font_family_frame_layout.addWidget(self.title_font_family_dropdown)
        #FONTS FRAME
        xfonts_frame = QFrame()
        xfonts_frame_layout = QHBoxLayout()
        xfonts_frame_layout.setContentsMargins(0,0,0,0)
        xfonts_frame.setLayout(xfonts_frame_layout)
        self.vbox.addWidget(xfonts_frame)
        ##FOR EACH TEXT
        #FONT SIZE SELECT
        xtitle_font_size_frame = QFrame()
        xtitle_font_size_frame_layout = QVBoxLayout()
        xtitle_font_size_frame_layout.setContentsMargins(0,0,0,0)
        xtitle_font_size_frame.setLayout(xtitle_font_size_frame_layout)
        xfonts_frame_layout.addWidget(xtitle_font_size_frame)
        xtitle_font_size_frame_layout.addWidget(QLabel("X-Axis Title Font Size"))
        self.xtitle_font_size_entry_box = QLineEdit()
        self.xtitle_font_size_entry_box.setValidator(QIntValidator(0, 100))
        self.xtitle_font_size_entry_box.setText(str(float(temp_dict["X-Axis Title font size"])*float(3/4)))
        xtitle_font_size_frame_layout.addWidget(self.xtitle_font_size_entry_box)
        #FONT FAMILY SELECT
        xtitle_font_family_frame = QFrame()
        xtitle_font_family_frame_layout = QVBoxLayout()
        xtitle_font_family_frame_layout.setContentsMargins(0,0,0,0)
        xtitle_font_family_frame.setLayout(xtitle_font_family_frame_layout)
        xfonts_frame_layout.addWidget(xtitle_font_family_frame)
        xtitle_font_family_frame_layout.addWidget(QLabel("X-Axis Title Font Family"))
        self.xtitle_font_family_dropdown = QComboBox()
        self.xtitle_font_family_dropdown.addItems(self.font_families)
        index = self.xtitle_font_family_dropdown.findText(temp_dict["X-Axis Title font family"])
        if index >= 0:
            self.xtitle_font_family_dropdown.setCurrentIndex(index)
        xtitle_font_family_frame_layout.addWidget(self.xtitle_font_family_dropdown)
        #FONTS FRAME
        yfonts_frame = QFrame()
        yfonts_frame_layout = QHBoxLayout()
        yfonts_frame_layout.setContentsMargins(0,0,0,0)
        yfonts_frame.setLayout(yfonts_frame_layout)
        self.vbox.addWidget(yfonts_frame)
        #FOR EACH TEXT
        #FONT SIZE SELECT
        ytitle_font_size_frame = QFrame()
        ytitle_font_size_frame_layout = QVBoxLayout()
        ytitle_font_size_frame_layout.setContentsMargins(0,0,0,0)
        ytitle_font_size_frame.setLayout(ytitle_font_size_frame_layout)
        yfonts_frame_layout.addWidget(ytitle_font_size_frame)
        ytitle_font_size_frame_layout.addWidget(QLabel("Y-Axis Title Font Size"))
        self.ytitle_font_size_entry_box = QLineEdit()
        self.ytitle_font_size_entry_box.setValidator(QIntValidator(0, 100))
        self.ytitle_font_size_entry_box.setText(str(float(temp_dict["Y-Axis Title font size"])*float(3/4)))
        ytitle_font_size_frame_layout.addWidget(self.ytitle_font_size_entry_box)
        #FONT FAMILY SELECT
        ytitle_font_family_frame = QFrame()
        ytitle_font_family_frame_layout = QVBoxLayout()
        ytitle_font_family_frame_layout.setContentsMargins(0,0,0,0)
        ytitle_font_family_frame.setLayout(ytitle_font_family_frame_layout)
        yfonts_frame_layout.addWidget(ytitle_font_family_frame)
        ytitle_font_family_frame_layout.addWidget(QLabel("Y-Axis Title Font Family"))
        self.ytitle_font_family_dropdown = QComboBox()
        self.ytitle_font_family_dropdown.addItems(self.font_families)
        index = self.ytitle_font_family_dropdown.findText(temp_dict["Y-Axis Title font family"])
        if index >= 0:
            self.ytitle_font_family_dropdown.setCurrentIndex(index)
        ytitle_font_family_frame_layout.addWidget(self.ytitle_font_family_dropdown)
        #FONTS FRAME
        xlfonts_frame = QFrame()
        xlfonts_frame_layout = QHBoxLayout()
        xlfonts_frame_layout.setContentsMargins(0,0,0,0)
        xlfonts_frame.setLayout(xlfonts_frame_layout)
        self.vbox.addWidget(xlfonts_frame)
        ##FOR EACH TEXT
        #FONT SIZE SELECT
        xltitle_font_size_frame = QFrame()
        xltitle_font_size_frame_layout = QVBoxLayout()
        xltitle_font_size_frame_layout.setContentsMargins(0,0,0,0)
        xltitle_font_size_frame.setLayout(xltitle_font_size_frame_layout)
        xlfonts_frame_layout.addWidget(xltitle_font_size_frame)
        xltitle_font_size_frame_layout.addWidget(QLabel("X-Axis Labels Font Size"))
        self.xltitle_font_size_entry_box = QLineEdit()
        self.xltitle_font_size_entry_box.setValidator(QIntValidator(0, 100))
        self.xltitle_font_size_entry_box.setText(str(float(temp_dict["X-Axis Labels font size"])*float(3/4)))
        xltitle_font_size_frame_layout.addWidget(self.xltitle_font_size_entry_box)
        #FONT FAMILY SELECT
        xltitle_font_family_frame = QFrame()
        xltitle_font_family_frame_layout = QVBoxLayout()
        xltitle_font_family_frame_layout.setContentsMargins(0,0,0,0)
        xltitle_font_family_frame.setLayout(xltitle_font_family_frame_layout)
        xlfonts_frame_layout.addWidget(xltitle_font_family_frame)
        xltitle_font_family_frame_layout.addWidget(QLabel("X-Axis Labels Font Family"))
        self.xltitle_font_family_dropdown = QComboBox()
        self.xltitle_font_family_dropdown.addItems(self.font_families)
        index = self.xltitle_font_family_dropdown.findText(temp_dict["X-Axis Labels font family"])
        if index >= 0:
            self.xltitle_font_family_dropdown.setCurrentIndex(index)
        xltitle_font_family_frame_layout.addWidget(self.xltitle_font_family_dropdown)
        #FONTS FRAME
        ylfonts_frame = QFrame()
        ylfonts_frame_layout = QHBoxLayout()
        ylfonts_frame_layout.setContentsMargins(0,0,0,0)
        ylfonts_frame.setLayout(ylfonts_frame_layout)
        self.vbox.addWidget(ylfonts_frame)
        #FOR EACH TEXT
        #FONT SIZE SELECT
        yltitle_font_size_frame = QFrame()
        yltitle_font_size_frame_layout = QVBoxLayout()
        yltitle_font_size_frame_layout.setContentsMargins(0,0,0,0)
        yltitle_font_size_frame.setLayout(yltitle_font_size_frame_layout)
        ylfonts_frame_layout.addWidget(yltitle_font_size_frame)
        yltitle_font_size_frame_layout.addWidget(QLabel("Y-Axis Labels Font Size"))
        self.yltitle_font_size_entry_box = QLineEdit()
        self.yltitle_font_size_entry_box.setValidator(QIntValidator(0, 100))
        self.yltitle_font_size_entry_box.setText(str(float(temp_dict["Y-Axis Labels font size"])*float(3/4)))
        yltitle_font_size_frame_layout.addWidget(self.yltitle_font_size_entry_box)
        #FONT FAMILY SELECT
        yltitle_font_family_frame = QFrame()
        yltitle_font_family_frame_layout = QVBoxLayout()
        yltitle_font_family_frame_layout.setContentsMargins(0,0,0,0)
        yltitle_font_family_frame.setLayout(yltitle_font_family_frame_layout)
        ylfonts_frame_layout.addWidget(yltitle_font_family_frame)
        yltitle_font_family_frame_layout.addWidget(QLabel("Y-Axis Labels Font Family"))
        self.yltitle_font_family_dropdown = QComboBox()
        self.yltitle_font_family_dropdown.addItems(self.font_families)
        index = self.yltitle_font_family_dropdown.findText(temp_dict["Y-Axis Labels font family"])
        if index >= 0:
            self.yltitle_font_family_dropdown.setCurrentIndex(index)
        yltitle_font_family_frame_layout.addWidget(self.yltitle_font_family_dropdown)
        #FONTS FRAME
        lfonts_frame = QFrame()
        lfonts_frame_layout = QHBoxLayout()
        lfonts_frame_layout.setContentsMargins(0,0,0,0)
        lfonts_frame.setLayout(lfonts_frame_layout)
        self.vbox.addWidget(lfonts_frame)
        #FOR EACH TEXT
        #FONT SIZE SELECT
        ltitle_font_size_frame = QFrame()
        ltitle_font_size_frame_layout = QVBoxLayout()
        ltitle_font_size_frame_layout.setContentsMargins(0,0,0,0)
        ltitle_font_size_frame.setLayout(ltitle_font_size_frame_layout)
        lfonts_frame_layout.addWidget(ltitle_font_size_frame)
        ltitle_font_size_frame_layout.addWidget(QLabel("Legend Title Font Size"))
        self.ltitle_font_size_entry_box = QLineEdit()
        self.ltitle_font_size_entry_box.setValidator(QIntValidator(0, 100))
        self.ltitle_font_size_entry_box.setText(str(float(temp_dict["Legend Title font size"])*float(3/4)))
        ltitle_font_size_frame_layout.addWidget(self.ltitle_font_size_entry_box)
        #FONT FAMILY SELECT
        ltitle_font_family_frame = QFrame()
        ltitle_font_family_frame_layout = QVBoxLayout()
        ltitle_font_family_frame_layout.setContentsMargins(0,0,0,0)
        ltitle_font_family_frame.setLayout(ltitle_font_family_frame_layout)
        lfonts_frame_layout.addWidget(ltitle_font_family_frame)
        ltitle_font_family_frame_layout.addWidget(QLabel("Legend Title Font Family"))
        self.ltitle_font_family_dropdown = QComboBox()
        self.ltitle_font_family_dropdown.addItems(self.font_families)
        index = self.ltitle_font_family_dropdown.findText(temp_dict["Legend Title font family"])
        if index >= 0:
            self.ltitle_font_family_dropdown.setCurrentIndex(index)
        ltitle_font_family_frame_layout.addWidget(self.ltitle_font_family_dropdown)
        #FONTS FRAME
        lefonts_frame = QFrame()
        lefonts_frame_layout = QHBoxLayout()
        lefonts_frame_layout.setContentsMargins(0,0,0,0)
        lefonts_frame.setLayout(lefonts_frame_layout)
        self.vbox.addWidget(lefonts_frame)
        #FOR EACH TEXT
        #FONT SIZE SELECT
        letitle_font_size_frame = QFrame()
        letitle_font_size_frame_layout = QVBoxLayout()
        letitle_font_size_frame_layout.setContentsMargins(0,0,0,0)
        letitle_font_size_frame.setLayout(letitle_font_size_frame_layout)
        lefonts_frame_layout.addWidget(letitle_font_size_frame)
        letitle_font_size_frame_layout.addWidget(QLabel("Legend Elements Font Size"))
        self.letitle_font_size_entry_box = QLineEdit()
        self.letitle_font_size_entry_box.setValidator(QIntValidator(0, 100))
        self.letitle_font_size_entry_box.setText(str(float(temp_dict["Legend Elements font size"])*float(3/4)))
        letitle_font_size_frame_layout.addWidget(self.letitle_font_size_entry_box)
        #FONT FAMILY SELECT
        letitle_font_family_frame = QFrame()
        letitle_font_family_frame_layout = QVBoxLayout()
        letitle_font_family_frame_layout.setContentsMargins(0,0,0,0)
        letitle_font_family_frame.setLayout(letitle_font_family_frame_layout)
        lefonts_frame_layout.addWidget(letitle_font_family_frame)
        letitle_font_family_frame_layout.addWidget(QLabel("Legend Elements Font Family"))
        self.letitle_font_family_dropdown = QComboBox()
        self.letitle_font_family_dropdown.addItems(self.font_families)
        index = self.letitle_font_family_dropdown.findText(temp_dict["Legend Elements font family"])
        if index >= 0:
            self.letitle_font_family_dropdown.setCurrentIndex(index)
        letitle_font_family_frame_layout.addWidget(self.letitle_font_family_dropdown)

 
    def save_changes(self):
        """When the user decides to save their Graph Preference changes, this function will take in their input and put it into a dictionay. This dictionary will then later be stored in a pickle file."""
        self.figure_dict = {}
        if not self.width_entry_box.text():
            self.figure_dict["width_inches"] = 5
        else:
            self.figure_dict["width_inches"] = int(self.width_entry_box.text())
        if not self.height_entry_box.text():
            self.figure_dict["height_inches"] = 5
        else:
            self.figure_dict["height_inches"] = int(self.height_entry_box.text())
        self.figure_dict["log_x"] = self.enable_log_x_axis_checkbox.isChecked()
        self.figure_dict["log_y"] = self.enable_log_y_axis_checkbox.isChecked()
        self.figure_dict["enable_error_bars"] = self.enable_error_bars_checkbox.isChecked()           
        
        #save selected font configs
        self.figure_dict["Title font family"] = self.title_font_family_dropdown.currentText()
        if self.title_font_size_entry_box.text() != "":
            self.figure_dict["Title font size"] = 4/3 * float(self.title_font_size_entry_box.text())
        self.figure_dict["X-Axis Title font family"] = self.xtitle_font_family_dropdown.currentText()
        if self.xtitle_font_size_entry_box.text() != "":
            self.figure_dict["X-Axis Title font size"] = 4/3 * float(self.xtitle_font_size_entry_box.text())
        self.figure_dict["Y-Axis Title font family"] = self.ytitle_font_family_dropdown.currentText()
        if self.ytitle_font_size_entry_box.text() != "":
            self.figure_dict["Y-Axis Title font size"] = 4/3 * float(self.ytitle_font_size_entry_box.text())
        self.figure_dict["X-Axis Labels font family"] = self.xltitle_font_family_dropdown.currentText()
        if self.xltitle_font_size_entry_box.text() != "":
            self.figure_dict["X-Axis Labels font size"] = 4/3 * float(self.xltitle_font_size_entry_box.text())
        self.figure_dict["Y-Axis Labels font family"] = self.yltitle_font_family_dropdown.currentText()
        if self.yltitle_font_size_entry_box.text() != "":
            self.figure_dict["Y-Axis Labels font size"] = 4/3 * float(self.yltitle_font_size_entry_box.text())
        self.figure_dict["Legend Title font family"] = self.ltitle_font_family_dropdown.currentText()
        if self.ltitle_font_size_entry_box.text() != "":
            self.figure_dict["Legend Title font size"] = 4/3 * float(self.ltitle_font_size_entry_box.text())
        self.figure_dict["Legend Elements font family"] = self.letitle_font_family_dropdown.currentText()
        if self.letitle_font_size_entry_box.text() != "":
            self.figure_dict["Legend Elements font size"] = 4/3 * float(self.letitle_font_size_entry_box.text())
        return self.figure_dict
