import sys
import os
import pathlib
import pickle
import PIL as pil
from PIL import Image
from PyQt5.QtSvg import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from PyQt5.QtGui import (QPixmap)
from pathvalidate import sanitize_filepath
import scipy.stats as sp
import plotly.express as px
from GraphKey.app.modules.data_export.export import *
from GraphKey.app.modules.data_import.utils import *
from GraphKey.app.modules.data_import.import_file import *
from GraphKey.app.modules.data_import import data_frame_functions
from GraphKey.app.modules.gui.window_elements import scrollable_checklist
from GraphKey.app.modules.gui.window_elements.edit_graph_frame import *
from GraphKey.app.modules.graphing.settings import *
from GraphKey.app.modules.graphing import generate_graph_pickle 
from GraphKey.app.modules.graphing.templates import *
from GraphKey.app.modules.graphing.figure_creation import *
from GraphKey.app.modules.data_import.graph_window_delegate import *
from GraphKey.app.modules.data_import.graph_icon_provider import *

class GraphCreationWindow(QWidget):
    """
    Class for the window in which graphs will be created and viewed.
    """
    #this is set to true when the bottom horizontal splitter frame has been deleted
    place_holder_frame_deleted = False
    #this is set to true when a new excel file is selected or a new sheet within that excel file is selected
    data_frame_changed = True
    #this is set to the previously selected data frame
    previous_data_frame = None
    # For creating and applying a template
    selected_pickle_file = None
   
    def __init__(self, project_path, app):
        """Initialize the UI for and set initial values for some class variables.
           :param: project_path - The path object for the project data that is going to be displayed in this window.
           :param: QApplication - Object used to run this program.
        """
        super().__init__()
        #set the project path
        self.project_path = project_path
        self.project_path_str = str(project_path)
        #init the UI
        self.initUI()
        self.app = app
        #Inits with initial workbook
        if self.workbook_select_dropdown.count() > 0:
            self.workbook_selected(self.workbook_select_dropdown.currentText())

    def initUI(self):        
        """Creates the main window as and places intital UI elements inside of it."""
        #SELF
        #create main window
        self.setGeometry(300, 150, 1342, 944)
        
        #SELF->MAIN_LAYOUT
        #create layout that will contain one main vertical splitter
        self.main_layout = QHBoxLayout(self)
        self.setLayout(self.main_layout)

        #SELF->MAIN_LAYOUT->MAIN_VERTICAL_SPLITTER
        #create vertical splitter which will contain two horizontal spitters
        self.main_vertical_splitter = QSplitter(Qt.Vertical)
        self.main_layout.addWidget(self.main_vertical_splitter)

        #SELF->MAIN_LAYOUT->MAIN_VERTICAL_SPLITTER->TOP_HORIZONTAL_SPLITTER
        #create top horizontal splitter which will contain file explorer, some dropdowns, some buttons, and the window for displaying the graph
        self.top_horizontal_splitter = QSplitter(Qt.Horizontal)
        self.main_vertical_splitter.addWidget(self.top_horizontal_splitter)

        #SELF->MAIN_LAYOUT->MAIN_VERTICAL_SPLITTER->BOTTOM_HORIZONTAL_SPLITTER
        #create bottom horizontal splitter which will contain scrollable checkbox list windows for selecting data columns/groups
        self.bottom_horizontal_splitter = QSplitter(Qt.Horizontal)        
        self.main_vertical_splitter.addWidget(self.bottom_horizontal_splitter)

        #SELF->MAIN_LAYOUT->MAIN_VERTICAL_SPLITTER->TOP_HORIZONTAL_SPLITTER->TOP_LEFT_FRAME
        #create QFrame and it's QVBoxLayout which will contain file explorer and some dropdowns
        self.top_left_frame = QFrame()
        self.top_left_frame_layout = QVBoxLayout()            
        self.top_left_frame_layout.setAlignment(Qt.AlignTop)
        self.top_left_frame.setLayout(self.top_left_frame_layout)        
        self.top_horizontal_splitter.addWidget(self.top_left_frame)
        
        #SELF->MAIN_LAYOUT->MAIN_VERTICAL_SPLITTER->TOP_HORIZONTAL_SPLITTER->IMAGE_FRAME
        self.image_frame = QFrame()
        self.image_frame_layout = QVBoxLayout()  
        self.image_frame_layout.setAlignment(Qt.AlignCenter)
        self.image_frame.setLayout(self.image_frame_layout)        
        self.top_horizontal_splitter.addWidget(self.image_frame)

        #SELF->MAIN_LAYOUT->MAIN_VERTICAL_SPLITTER->TOP_HORIZONTAL_SPLITTER->TOP_RIGHT_FRAME
        #create QFrame and it's QVBoxLayout which contains edit graph frame
        self.top_right_frame = QFrame()
        self.top_right_frame_layout = QVBoxLayout()            
        self.top_right_frame.setLayout(self.top_right_frame_layout)        
        self.top_horizontal_splitter.addWidget(self.top_right_frame)

        #SELF->MAIN_LAYOUT->MAIN_VERTICAL_SPLITTER->TOP_HORIZONTAL_SPLITTER->TOP_LEFT_FRAME->MISC DROPDOWNS ABOVE THE FILE TREE VIEW
        #Workbook Selection
        self.workbook_select_label = QLabel("Select Workbook")
        self.workbook_select_dropdown = QComboBox()        
        #Add folders to dropdown
        self.workbook_folders = next(os.walk(self.project_path_str))[1]
        self.workbook_select_dropdown.addItems(self.workbook_folders)
        self.workbook_select_dropdown.activated[str].connect(self.workbook_selected)
        self.workbook_select_dropdown.setFixedHeight(20)
        self.top_left_frame_layout.addWidget(self.workbook_select_label)
        self.top_left_frame_layout.addWidget(self.workbook_select_dropdown)
        #Select Workbook Edition
        self.workbook_edition_label = QLabel("Select Workbook Edition")
        self.workbook_edition_dropdown = QComboBox()
        self.workbook_edition_dropdown.activated[str].connect(self.edition_dropdown_clicked)
        self.workbook_edition_dropdown.setFixedHeight(20)
        self.top_left_frame_layout.addWidget(self.workbook_edition_label)
        self.top_left_frame_layout.addWidget(self.workbook_edition_dropdown)
        #Select Sheet
        self.sheet_select_label = QLabel("Select Excel Sheet")
        self.sheet_select_dropdown = QComboBox()
        self.sheet_select_dropdown.activated[str].connect(self.sheet_selected)
        self.sheet_select_dropdown.setFixedHeight(20)
        self.top_left_frame_layout.addWidget(self.sheet_select_label)
        self.top_left_frame_layout.addWidget(self.sheet_select_dropdown)
        #Sheet Graphs Label 
        self.sheet_graph_label = QLabel("Sheet Graphs")
        self.top_left_frame_layout.addWidget(self.sheet_graph_label)

        #SELF->MAIN_LAYOUT->MAIN_VERTICAL_SPLITTER->TOP_HORIZONTAL_SPLITTER->TOP_LEFT_FRAME->FILE_TREE_VIEW
        #create file system model for tree view
        self.file_tree_model = QFileSystemModel(self)     
        self.file_tree_model.setRootPath(self.project_path_str)
        filter_list = ["*.xlsx", "*.xlsm", "*.png", "*.pickle"]
        self.file_tree_model.setNameFilters(filter_list)
        self.file_tree_model.setNameFilterDisables(False)
        #create QTreeView from the file system model and add to top horizontal splitter
        self.file_tree_view = QTreeView()
        self.file_tree_view.setModel(self.file_tree_model)
        #hide all columns except file name
        self.file_tree_view.setColumnHidden(1, True)
        self.file_tree_view.setColumnHidden(2, True)
        self.file_tree_view.setColumnHidden(3, True)
        #hide headers
        self.file_tree_view.setHeaderHidden(True) 
        #hide border
        self.file_tree_view.setFrameShadow(QFrame.Raised)
        self.file_tree_view.setExpanded(self.file_tree_model.index(self.project_path_str, 0), True)
        self.file_tree_view.header().setStretchLastSection(True)
        #set signal for when tree view is clicked
        self.file_tree_view.clicked.connect(self.pickle_clicked)
        self.file_tree_view.setStyleSheet("border: 1px solid black")
        self.top_left_frame_layout.addWidget(self.file_tree_view)
        #adds delegate to remove file extensions
        self.file_tree_delegate = GraphWindowDelegate(self.file_tree_view)
        self.file_tree_view.setItemDelegate(self.file_tree_delegate)
        #sets graph icons to a pretty little graph
        self.file_tree_model.setIconProvider(GraphIconProvider())
        
        #SELF->MAIN_LAYOUT->MAIN_VERTICAL_SPLITTER->TOP_HORIZONTAL_SPLITTER->TOP_LEFT_FRAME->MISC DROPDOWNS/BUTTONS BELOW THE FILE TREE VIEW
        #create dropdowns for selecting an excel sheet and a type of graph as well as generate graphs button
        #Graph Type selection
        self.graph_type_select_label = QLabel("Graph Type")
        self.graph_type_select_dropdown = QComboBox()
        self.graph_type_select_dropdown.activated[str].connect(self.graph_type_selected)
        self.graph_type_select_dropdown.setFixedHeight(20)
        self.graph_type_select_dropdown.addItems(["Scatter Plot", "Bar Graph", "Box Plot"])
        self.selected_graph_type = 'Scatter Plot'
        self.generate_graphs_button = QPushButton("Generate Graphs")
        self.generate_graphs_button.clicked.connect(self.generate_graphs)
        self.generate_graphs_button.resize(self.generate_graphs_button.sizeHint())
        self.top_left_frame_layout.addWidget(self.graph_type_select_label)
        self.top_left_frame_layout.addWidget(self.graph_type_select_dropdown)
        self.top_left_frame_layout.addStretch(1) 
        self.top_left_frame_layout.addWidget(self.generate_graphs_button)        
        #Generate graph button
        self.generate_graphs_button.clicked.connect(self.generate_graphs)
        self.generate_graphs_button.resize(self.generate_graphs_button.sizeHint())
        
        
        #SELF->MAIN_LAYOUT->MAIN_VERTICAL_SPLITTER->TOP_HORIZONTAL_SPLITTER->IMAGE_FRAME->SVG_DISPLAY
        #create QSvgWidget to display image and add to top horizontal splitter
        self.svg_display = QSvgWidget()
        self.image_frame_layout.addWidget(self.svg_display)   

        
        #SELF->MAIN_LAYOUT->MAIN_VERTICAL_SPLITTER->TOP_HORIZONTAL_SPLITTER->IMAGE_FRAME->SCATTER_INFO_LABEL
        self.scatter_info_label = QLabel()
        self.scatter_info_label.hide()
        self.image_frame_layout.addWidget(self.scatter_info_label)       
                           
        #SELF->MAIN_LAYOUT->MAIN_VERTICAL_SPLITTER->TOP_HORIZONTAL_SPLITTER->TOP_RIGHT_FRAME->EDIT_GRAPH_FRAME
        self.edit_graph_frame = EditGraphFrame(self)
        self.edit_graph_frame.setMinimumWidth(285)
        self.edit_graph_frame.changes_saved.connect(self.save_graph_changes_to_pickle)
        self.top_right_frame_layout.addWidget(self.edit_graph_frame)
         
        #SELF->MAIN_LAYOUT->MAIN_VERTICAL_SPLITTER->BOTTOM_HORIZONTAL_SPLITTER->Scatter UI Elements
        #add scatter UI selections to the window but hide them
        self.scatter_x_var_select = scrollable_checklist.ScrollableCheckList(self.bottom_horizontal_splitter, "X-Variable", self)
        self.scatter_x_var_select.resize(self.height(), 500)        
        self.scatter_y_var_select = scrollable_checklist.ScrollableCheckList(self.bottom_horizontal_splitter, "Y-Variable", self)
        self.scatter_y_var_select.resize(self.height(), 500)
        self.scatter_color_group_select = scrollable_checklist.ScrollableCheckList(self.bottom_horizontal_splitter, "Color Grouping Variables", self)
        self.scatter_color_group_select.resize(self.height(), 500)   
        self.scatter_marker_group_select = scrollable_checklist.ScrollableCheckList(self.bottom_horizontal_splitter, "Marker Grouping Variables", self)
        self.bottom_horizontal_splitter.addWidget(self.scatter_x_var_select)        
        self.bottom_horizontal_splitter.addWidget(self.scatter_y_var_select)  
        self.bottom_horizontal_splitter.addWidget(self.scatter_color_group_select)
        self.bottom_horizontal_splitter.addWidget(self.scatter_marker_group_select)
        self.scatter_x_var_select.hide()
        self.scatter_y_var_select.hide()
        self.scatter_color_group_select.hide()
        self.scatter_marker_group_select.hide()

        #SELF->MAIN_LAYOUT->MAIN_VERTICAL_SPLITTER->BOTTOM_HORIZONTAL_SPLITTER->Bar UI Elements
        self.bar_group_select = scrollable_checklist.ScrollableCheckList(self.bottom_horizontal_splitter, "Groupings", self)
        self.bar_group_select.resize(self.height(), 500)
        self.bottom_horizontal_splitter.addWidget(self.bar_group_select)
        self.bar_group_filtering_select = scrollable_checklist.ScrollableCheckList(self.bottom_horizontal_splitter, "Group Filtering", self)
        self.bar_group_filtering_select.resize(self.height(), 500)        
        self.bottom_horizontal_splitter.addWidget(self.bar_group_filtering_select)
        self.bar_sub_group_select = scrollable_checklist.ScrollableCheckList(self.bottom_horizontal_splitter, "Sub-Groupings", self)
        self.bar_sub_group_select.resize(self.height(), 500)
        self.bottom_horizontal_splitter.addWidget(self.bar_sub_group_select)
        self.bar_y_var_select = scrollable_checklist.ScrollableCheckList(self.bottom_horizontal_splitter, "Y-Variable", self)
        self.bar_y_var_select.resize(self.height(), 500)
        self.bottom_horizontal_splitter.addWidget(self.bar_y_var_select)
        self.bar_group_select.hide()
        self.bar_group_filtering_select.hide()
        self.bar_sub_group_select.hide()
        self.bar_y_var_select.hide()

        #SELF->MAIN_LAYOUT->MAIN_VERTICAL_SPLITTER->BOTTOM_HORIZONTAL_SPLITTER->Box plot UI Elements
        self.box_group_select = scrollable_checklist.ScrollableCheckList(self.bottom_horizontal_splitter, "Groupings", self)
        self.box_group_select.resize(self.height(), 500)
        self.bottom_horizontal_splitter.addWidget(self.box_group_select)
        self.box_group_filtering_select = scrollable_checklist.ScrollableCheckList(self.bottom_horizontal_splitter, "Group Filtering", self)
        self.box_group_filtering_select.resize(self.height(), 500)        
        self.bottom_horizontal_splitter.addWidget(self.box_group_filtering_select)
        self.box_y_var_select = scrollable_checklist.ScrollableCheckList(self.bottom_horizontal_splitter, "Y-Variable", self)
        self.box_y_var_select.resize(self.height(), 500)
        self.bottom_horizontal_splitter.addWidget(self.box_y_var_select)
        self.box_color_group_select = scrollable_checklist.ScrollableCheckList(self.bottom_horizontal_splitter, "Color Grouping Variables", self)
        self.box_color_group_select.resize(self.height(), 500)  
        self.bottom_horizontal_splitter.addWidget(self.box_color_group_select)
        self.box_group_select.hide()
        self.box_group_filtering_select.hide()
        self.box_y_var_select.hide()
        self.box_color_group_select.hide()

        #add a QFrame to the bottom splitter so it isn't empty when to data has been selected
        self.place_holder_frame = QFrame()                        
        self.bottom_horizontal_splitter.addWidget(self.place_holder_frame) 
        
        #set default sizes for splitters
        self.main_vertical_splitter.setSizes([535, 409])
        self.top_horizontal_splitter.setSizes([323, 525, 494])
        
        if self.workbook_select_dropdown.count() is 0: 
            self.top_left_frame.setEnabled(False)
            self.top_right_frame.setEnabled(False)
            
    def workbook_selected(self, text):
        """Gets the selected workbook from the workbook select dropdown. Assigns a global variable for the the folder of the workbook that has been selected. Updates the workbook edition dropdown. Calls self.edition_dropdown_clicked which subsequently updates the sheets dropdown.
           :param: text - The text for the selected dropdown item.
        """
        self.workbookFolderSelected = self.project_path_str+"/"+text

        self.editions = next(os.walk(self.workbookFolderSelected+"/editions/"))[1]
        self.workbook_edition_dropdown.clear()
        for i in range (1, len(self.editions)+1):
            self.workbook_edition_dropdown.addItem(str(i))

        if self.workbook_edition_dropdown.count() > 0:
            self.edition_dropdown_clicked(self.workbook_edition_dropdown.currentText())


    def sheet_selected(self, text):
        """Gets the selected sheet from the sheet select dropdown. Assigns the global variable for the selected data frame to the data frame corresponding with that sheet.
           :param: text - The text for the selected dropdown item.
        """
        #get the data frame for this sheet        
        self.selected_sheet_df = self.sheets.get(text)
        self.sheetSelected = text
        #check if there is a new data frame then the previously selected dataframe
        self.data_frame_changed = not(data_frame_functions.compare_data_frames(self.selected_sheet_df, self.previous_data_frame))
        self.previous_data_frame = self.selected_sheet_df
        #update what's in the sheet view
        self.file_tree_view.setRootIndex(self.file_tree_model.index(self.workbookFolderSelected+"/sheets/"+text))   
        self.sheetSelected = text
        #if data frame changed, repopulate UI elements with new data
        if self.data_frame_changed:
            self.populate_all_ui_elements()   
        #populate the data frame selection UI elements for this sheet
        self.show_selected_graph_type_ui_elements()


    def graph_type_selected(self, text):
        """Gets the selected graph type and calls a method to update the UI accordingly.
           :param: text - The text for the selected dropdown item.
        """
        #get the selected graph type
        self.selected_graph_type = text        
        #show the UI elements for that graph type
        self.show_selected_graph_type_ui_elements()


    def edition_dropdown_clicked(self, text):
        """Updates the sheets dictionary based on the edition selection. Also updates the sheets selection dropdown based on the selection made in this.
           :param: text - The text for the selected dropdown item.
        """
        editionPicklesFolder = self.workbookFolderSelected + "/editions/" + text + "/"+ text + "-pickles"
        if os.path.isdir(editionPicklesFolder):
            self.selected_dataset_folder = pathlib.Path(self.workbookFolderSelected)
            #get a dictionary of pandas data frames from all sheets in this excel workbook
            self.sheets = DataImport().import_pickled_dfs_from_folder(editionPicklesFolder) 
            #clear the sheet select dropdown
            self.sheet_select_dropdown.clear()
            #populate it with the new sheet data
            self.sheet_select_dropdown.addItems(self.sheets)
            #set the selected dataframe to be the default first item in the dropdown
            first_sheet_name = next(iter(self.sheets))
            #Sets the default path in the treeview
            self.sheetSelected = first_sheet_name
            self.file_tree_view.setRootIndex(self.file_tree_model.index(self.workbookFolderSelected+"/sheets/"+first_sheet_name))   
            self.selected_sheet_df = self.sheets.get(first_sheet_name)
            #check if there is a new data frame then the previously selected dataframe
            self.data_frame_changed = not(data_frame_functions.compare_data_frames(self.selected_sheet_df, self.previous_data_frame))
            self.previous_data_frame = self.selected_sheet_df
            #if data frame changed, repopulate UI elements with new data
            if self.data_frame_changed:
                    self.populate_all_ui_elements()                
            #show data selection UI elements in for the selected graph type
            self.show_selected_graph_type_ui_elements()
            #if there is a pickle file selected, update it to have the new data
            if self.file_tree_model.fileInfo(self.file_tree_view.currentIndex()).absoluteFilePath() is not "": 
                self.pickle_clicked(self.file_tree_view.currentIndex())
    
    def pickle_clicked(self, index):
        """Loads in the figure from the pickle file selected and displays it in the UI.
           :param: The index of the item in the file_tree_model which can be used to get information about the selected file.
        """
        if self.file_tree_model.fileInfo(index).isFile():
            fpath = self.file_tree_model.fileInfo(index).absoluteFilePath()
            figure_dict = generate_graph_pickle.getDataFromPickle(fpath)
            df = None
            #try:
            if figure_dict["graph_type"] == "scatter":
                figure = create_scatter_graph(self.sheets.get(figure_dict["sheet_selected"]), figure_dict)
                self.edit_graph_frame.hide_bar_graph_options()
                self.edit_graph_frame.show_scatter_graph_options()
                self.scatter_info_label.show()
                self.scatter_info_label.setText(self.get_scatter_info(figure_dict, figure, self.sheets.get(figure_dict["sheet_selected"])))
            elif figure_dict["graph_type"] == "bar":
                figure = create_bar_graph(self.sheets.get(figure_dict["sheet_selected"]), figure_dict)
                self.edit_graph_frame.show_bar_graph_options()
                self.edit_graph_frame.hide_scatter_graph_options()
                self.scatter_info_label.hide()
            elif figure_dict["graph_type"] == "box":
                figure = create_box_graph(self.sheets.get(figure_dict["sheet_selected"]), figure_dict)
                self.edit_graph_frame.hide_bar_graph_options()
                self.edit_graph_frame.hide_scatter_graph_options()
                self.scatter_info_label.hide()
            figure.write_image(path_globals.tmp_folder_path_str + "temp.svg", format = 'svg', scale = figure_dict["scale_factor"], width=figure_dict["width"], height=figure_dict["height"])            
            self.display_svg(path_globals.tmp_folder_path_str + "temp.svg", figure_dict["width"], figure_dict["height"])
            self.edit_graph_frame.populate_dict_data(figure_dict)
            self.selected_pickle_file = fpath

    def single_graph_generation_for_export(self, fpath):
        """Generates a single graphs. Mainly used for exporting the graph.
           :param: fpath - Filepath to the graph pickle the graph will be generated from.
           :return: A figure and figure dictionary.
        """
        figure_dict = generate_graph_pickle.getDataFromPickle(fpath)
        figure = None
        df = None
        try:
            if figure_dict["graph_type"] == "scatter":
                figure = create_scatter_graph(self.sheets.get(figure_dict["sheet_selected"]), figure_dict)
            elif figure_dict["graph_type"] == "bar":
                figure = create_bar_graph(self.sheets.get(figure_dict["sheet_selected"]), figure_dict)
            elif figure_dict["graph_type"] == "box":
                figure = create_box_graph(self.sheets.get(figure_dict["sheet_selected"]), figure_dict)
        except Exception as e:
            self.show_alert("Wrong data set selected")
            return None
        return figure, figure_dict


    def show_alert(self, text):
        """Shows a basic alert box"""
        alert = QMessageBox()
        alert.setText(text)
        alert.setStandardButtons(QMessageBox.Ok)
        alert.exec_()


    def group_checkbox_state_changed(self):
        """Adds or removes the group filtering checkboxes associated with this group if its checkbox state changes."""
        selected_group_column = self.sender().text()
        if self.selected_graph_type == 'Bar Graph':
            if (self.sender().isChecked()):
                unique_col_values = self.group_filtering_dict[selected_group_column]
                self.bar_group_filtering_select.addGroupFilterCheckboxes(selected_group_column, unique_col_values)
                self.bar_sub_group_select.hideAndUncheckCheckBox(selected_group_column)
            else:
                self.bar_group_filtering_select.removeCheckBoxesOfGroupFilterColumn(selected_group_column)
                self.bar_sub_group_select.showCheckBox(selected_group_column)
        elif self.selected_graph_type == 'Box Plot':
            if (self.sender().isChecked()):
                unique_col_values = self.group_filtering_dict[selected_group_column]
                self.box_group_filtering_select.addGroupFilterCheckboxes(selected_group_column, unique_col_values)
            else:
                self.box_group_filtering_select.removeCheckBoxesOfGroupFilterColumn(selected_group_column)


    def generate_graphs(self):
        """Function called when "Generate Graphs" button is pressed. Creates graphs based on UI selections."""
        self.msg = QMessageBox()
        self.msg.setWindowTitle("Generating Graphs")
        self.msg.setText("Please wait while the graphs are generated. Do not close this window.")
        self.msg.setModal(True)
        self.app.processEvents()
        self.msg.show()
        self.app.processEvents()

        if self.selected_graph_type == 'Bar Graph':
            self.generate_bar_graphs()
        elif self.selected_graph_type == 'Scatter Plot':
            self.generate_scatter_graphs()  
        elif self.selected_graph_type == 'Box Plot':
            self.generate_box_graphs()

    def settings(self):
        """Displays the Default Graph Preferences window."""
        setting = Settings(0) 
        setting.show() 
        
    def generate_scatter_graphs(self):
        """Creates scatter graph files based on the selected UI elements."""
        #get list of variables to determine coloring of dots
        selected_color_grouping_variables = self.scatter_color_group_select.getListOfCheckedCheckBoxes()       
        #get list of variables to determine marker style of dots
        selected_marker_variables = self.scatter_marker_group_select.getListOfCheckedCheckBoxes()
        #get list of x-axis variables
        selected_x_variables = self.scatter_x_var_select.getListOfCheckedCheckBoxes()
        #get list of y-axis variables
        selected_y_variables = self.scatter_y_var_select.getListOfCheckedCheckBoxes()
        #get color discrete sequence
        color_configs = pickle.load(open(Path(path_globals.settings_folder_path / "save_color_configs.p"), "rb"))
        color_list = color_configs["current_theme"]["colors"] 
        #get symbol sequence
        symbol_configs = pickle.load(open(Path(path_globals.settings_folder_path / "save_shape_configs.p"), "rb"))
        symbol_list = symbol_configs["current_theme"]["shapes"] 
        #get other configs
        other_configs = pickle.load(open(Path(path_globals.settings_folder_path / "save_other_configs.p"), "rb"))
        #iterate through all variable combinations and create graphs
        for x_var in selected_x_variables:
            for y_var in selected_y_variables:
                figure_dict = {"x":x_var, "y":y_var, "color_grouping_variables":selected_color_grouping_variables, \
                        "marker_style_variables":selected_marker_variables, "color_discrete_sequence":color_list, \
                        "symbol_sequence":symbol_list, "width_inches":other_configs["width_inches"], \
                        "height_inches":other_configs["height_inches"], "log_x":other_configs["log_x"], \
                        "log_y":other_configs["log_y"], \
                        'Title font family':other_configs["Title font family"], 'Title font size':other_configs["Title font size"], \
                        'X-Axis Title font family':other_configs["X-Axis Title font family"], 'X-Axis Title font size':other_configs["X-Axis Title font size"], \
                        'Y-Axis Title font family':other_configs["Y-Axis Title font family"], 'Y-Axis Title font size':other_configs["Y-Axis Title font size"], \
                        'X-Axis Labels font family':other_configs["X-Axis Labels font family"], 'X-Axis Labels font size':other_configs["X-Axis Labels font size"], \
                        'Y-Axis Labels font family': other_configs["Y-Axis Labels font family"], 'Y-Axis Labels font size':other_configs["Y-Axis Labels font size"], \
                        'Legend Title font family':other_configs["Legend Title font family"], 'Legend Title font size':other_configs["Legend Title font size"], \
                        'Legend Elements font family':other_configs["Legend Elements font family"], 'Legend Elements font size':other_configs["Legend Elements font size"]}
                figure = create_scatter_graph(self.selected_sheet_df, figure_dict)
                #create image file name
                if len(selected_color_grouping_variables) > 0:
                    graph_image_file_name = "Scatter x-" + x_var + " y-" + y_var + " color_group-" + ' and '.join(selected_color_grouping_variables) + ".pickle"
                else:
                    graph_image_file_name = "Scatter x-" + x_var + " y-" + y_var + ".pickle"
                graph_image_file_name = self.replace_invalid_file_chars(graph_image_file_name)
                graph_image_file_name = sanitize_filepath(graph_image_file_name)                
                file_path = self.selected_dataset_folder / "sheets" / self.sheetSelected / graph_image_file_name
                file_path_str = str(file_path)
                # Save to pickle file
                folder_path = self.selected_dataset_folder / "sheets" / self.sheetSelected
                folder_path_str = str(folder_path)
                generate_graph_pickle.createScatterGraphPickle(file_path_str, self.sheetSelected, figure_dict)
        self.msg.close()

    def generate_bar_graphs(self):
        """Creates bar graph files based on the selected UI elements."""
        #get dictionary that maps a column name to the values that should be filtered out of it
        filter_dict = self.bar_group_filtering_select.getListOfFilterCheckedCheckBoxesDict()
        #get list of selected groups
        group_list = self.bar_group_select.getListOfCheckedCheckBoxes()
        #get list of selected sub-groups if any
        sub_group_list = self.bar_sub_group_select.getListOfCheckedCheckBoxes()
        #get list of selected y variables
        selected_y_variables = self.bar_y_var_select.getListOfCheckedCheckBoxes()
        #get color discrete sequence
        color_configs = pickle.load(open(Path(path_globals.settings_folder_path / "save_color_configs.p"), "rb"))
        color_list = color_configs["current_theme"]["colors"]  
        #get other configs
        other_configs = pickle.load(open(Path(path_globals.settings_folder_path / "save_other_configs.p"), "rb"))
        #iterate through all selected y_vars and create graphs
        for y_var in selected_y_variables:
            figure_dict = {"group_list": group_list, "sub_group_list":sub_group_list, "x": ' and '.join(group_list), "y":y_var, "color_discrete_sequence":color_list, \
                        "filter_dict":filter_dict, "width_inches":other_configs["width_inches"], "height_inches":other_configs["height_inches"], \
                        "log_x":other_configs["log_x"], "log_y":other_configs["log_y"], "enable_error_bars":other_configs["enable_error_bars"], \
                        'Title font family':other_configs["Title font family"], 'Title font size':other_configs["Title font size"], \
                        'X-Axis Title font family':other_configs["X-Axis Title font family"], 'X-Axis Title font size':other_configs["X-Axis Title font size"], \
                        'Y-Axis Title font family':other_configs["Y-Axis Title font family"], 'Y-Axis Title font size':other_configs["Y-Axis Title font size"], \
                        'X-Axis Labels font family':other_configs["X-Axis Labels font family"], 'X-Axis Labels font size':other_configs["X-Axis Labels font size"], \
                        'Y-Axis Labels font family': other_configs["Y-Axis Labels font family"], 'Y-Axis Labels font size':other_configs["Y-Axis Labels font size"], \
                        'Legend Title font family':other_configs["Legend Title font family"], 'Legend Title font size':other_configs["Legend Title font size"], \
                        'Legend Elements font family':other_configs["Legend Elements font family"], 'Legend Elements font size':other_configs["Legend Elements font size"]}
            figure = create_bar_graph(self.selected_sheet_df, figure_dict)
            if len(sub_group_list) != 0:
                graph_image_file_name = "Bar x-" + ' and '.join(group_list) + " y-" + y_var + " subsgroup- " + ' and '.join(sub_group_list) + ".pickle"
            else:
                graph_image_file_name = "Bar x-" + ' and '.join(group_list) + " y-" + y_var + ".pickle"
            graph_image_file_name = self.replace_invalid_file_chars(graph_image_file_name)
            graph_image_file_name = sanitize_filepath(graph_image_file_name)
            file_path = self.selected_dataset_folder / "sheets" / self.sheetSelected / graph_image_file_name
            file_path_str = str(file_path)
            # Save to pickle file
            folder_path = self.selected_dataset_folder / "sheets" / self.sheetSelected
            folder_path_str = str(folder_path)            
            generate_graph_pickle.createBarGraphPickle(file_path_str, self.sheetSelected, figure_dict)
        self.msg.close()

    def generate_box_graphs(self):
        """Creates box graph files based on the selected UI elements."""
        #get dictionary that maps a column name to the values that should be filtered out of it
        filter_dict = self.box_group_filtering_select.getListOfFilterCheckedCheckBoxesDict()
        #get list of selected groups
        group_list = self.box_group_select.getListOfCheckedCheckBoxes()        
        #get list of selected y variables
        selected_y_variables = self.box_y_var_select.getListOfCheckedCheckBoxes()
        #get list of color grouping variables
        selected_color_grouping_variables = self.box_color_group_select.getListOfCheckedCheckBoxes()
        #get color discrete sequence
        color_configs = pickle.load(open(Path(path_globals.settings_folder_path / "save_color_configs.p"), "rb"))
        color_list = color_configs["current_theme"]["colors"]  
        #get other configs
        other_configs = pickle.load(open(Path(path_globals.settings_folder_path / "save_other_configs.p"), "rb"))
        #iterate through all selected y_vars and create graphs
        for y_var in selected_y_variables:
            figure_dict = {"group_list": group_list, "x": ' and '.join(group_list), "y":y_var, "color_grouping_variables":selected_color_grouping_variables, "color_discrete_sequence":color_list, \
                        "filter_dict":filter_dict, "width_inches":other_configs["width_inches"], "height_inches":other_configs["height_inches"], \
                        "log_x":other_configs["log_x"], "log_y":other_configs["log_y"], \
                        'Title font family':other_configs["Title font family"], 'Title font size':other_configs["Title font size"], \
                        'X-Axis Title font family':other_configs["X-Axis Title font family"], 'X-Axis Title font size':other_configs["X-Axis Title font size"], \
                        'Y-Axis Title font family':other_configs["Y-Axis Title font family"], 'Y-Axis Title font size':other_configs["Y-Axis Title font size"], \
                        'X-Axis Labels font family':other_configs["X-Axis Labels font family"], 'X-Axis Labels font size':other_configs["X-Axis Labels font size"], \
                        'Y-Axis Labels font family': other_configs["Y-Axis Labels font family"], 'Y-Axis Labels font size':other_configs["Y-Axis Labels font size"], \
                        'Legend Title font family':other_configs["Legend Title font family"], 'Legend Title font size':other_configs["Legend Title font size"], \
                        'Legend Elements font family':other_configs["Legend Elements font family"], 'Legend Elements font size':other_configs["Legend Elements font size"]}
            figure = create_box_graph(self.selected_sheet_df, figure_dict)#, color_discrete_sequence=color_list)
            graph_image_file_name = "Box Plot x-" + ' and '.join(group_list) + " y-" + y_var + ".pickle"
            graph_image_file_name = self.replace_invalid_file_chars(graph_image_file_name)
            graph_image_file_name = sanitize_filepath(graph_image_file_name)
            file_path = self.selected_dataset_folder / "sheets" / self.sheetSelected / graph_image_file_name
            file_path_str = str(file_path)
            #figure.write_image(file = file_path_str, format = 'png', scale = 1, width = 500, height = 500, validate = True)
            # Save to pickle file
            folder_path = self.selected_dataset_folder / "sheets" / self.sheetSelected
            folder_path_str = str(folder_path)            
            generate_graph_pickle.createBoxGraphPickle(file_path_str, self.sheetSelected, figure_dict)  
        self.msg.close()

    def save_graph_changes_to_pickle(self):
        """Saves changes specified in the edit graph frame to the pickle file for that figure."""
        figure_dict = self.edit_graph_frame.figure_dict
        if figure_dict["graph_type"] == "scatter":
            generate_graph_pickle.createScatterGraphPickle(self.selected_pickle_file, self.sheetSelected, figure_dict)
        elif figure_dict["graph_type"] == "bar":
            generate_graph_pickle.createBarGraphPickle(self.selected_pickle_file, self.sheetSelected, figure_dict)
        elif figure_dict["graph_type"] == "box":
            generate_graph_pickle.createBoxGraphPickle(self.selected_pickle_file, self.sheetSelected, figure_dict)
        self.pickle_clicked(self.file_tree_view.currentIndex())

    def replace_invalid_file_chars(self, file_str:str):
        """Returns a copy of "file_str" with characters that are invalid in a file name replaced.
           :param: file_str - The str containing the file name.
           :return: A string without characters invalid in a file name.
        """
        file_str = file_str.replace('/', ' per ')
        return file_str

    def updateImportDropdown(self, text):
        """Adds an item to import dropdown. Used by main_window after an import finishes.
           :param: text - The text of the item to be added to the workbook dropdown.
        """
        self.workbook_select_dropdown.addItem(text)
        self.workbook_selected(text)

    def updateEditionDropdown(self, text):
        """Adds an edition to the edition dropdown. Used by main window. 
           :param: text - The text of the item to be added to the edition dropdown.
        """
        self.workbook_edition_dropdown.addItem(text)
        
    def show_selected_graph_type_ui_elements(self):
        """Populates the UI with certain elements depending on the selected graph type."""
        #remove the placeholder frame if it hasn't been deleted yet        
        if self.place_holder_frame_deleted != True:
            self.place_holder_frame.deleteLater()
            self.place_holder_frame.hide()
            self.place_holder_frame_deleted = True        
        #hide/show UI elements for selected type
        if self.selected_graph_type == 'Scatter Plot':
            self.show_scatter_ui_elements()
            self.hide_box_ui_elements()
            self.hide_bar_ui_elements()
        elif self.selected_graph_type == 'Bar Graph':
            self.show_bar_ui_elements()
            self.hide_scatter_ui_elements()
            self.hide_box_ui_elements()            
        elif self.selected_graph_type == 'Box Plot':
            self.show_box_ui_elements()
            self.hide_scatter_ui_elements()
            self.hide_bar_ui_elements()

    def hide_bar_ui_elements(self):
        """Hides the UI elements that Bar Graphs don't use."""
        self.bar_group_select.hide()
        self.bar_group_filtering_select.hide()
        self.bar_sub_group_select.hide()
        self.bar_y_var_select.hide()

    def hide_box_ui_elements(self):
        """Hides the UI elements that Box Plots don't use."""
        self.box_group_select.hide()
        self.box_group_filtering_select.hide()
        self.box_y_var_select.hide()   
        self.box_color_group_select.hide()

    def hide_scatter_ui_elements(self):
        """Hides the UI elements that Scatter Plots don't use."""
        self.scatter_x_var_select.hide()            
        self.scatter_y_var_select.hide()
        self.scatter_color_group_select.hide()
        self.scatter_marker_group_select.hide()

    def show_bar_ui_elements(self):
        """Shows the UI elements that Bar Graphs use."""
        self.bar_group_select.show()
        self.bar_group_filtering_select.show()
        self.bar_sub_group_select.show()
        self.bar_y_var_select.show()

    def show_box_ui_elements(self):
        """Shows the UI elements that Box Plots use."""
        self.box_group_select.show()
        self.box_group_filtering_select.show()
        self.box_y_var_select.show()
        self.box_color_group_select.show()

    def show_scatter_ui_elements(self):
        """Shows the UI elements that Scatter Plots use."""
        self.scatter_x_var_select.show()            
        self.scatter_y_var_select.show()
        self.scatter_color_group_select.show()
        self.scatter_marker_group_select.show()

    def populate_all_ui_elements(self):
        """Adds the checkboxes to the scatter, bar, and box plot UI elements. Does NOT show any of them though."""
        self.populate_scatter_plot_ui_elements()
        self.populate_bar_plot_ui_elements()
        self.populate_box_plot_ui_elements()
        #set equal spacing for all elements in the bottom horizontal splitter
        self.bottom_horizontal_splitter.setSizes([500, 500, 500, 500, 500, 500, 500, 500, 500, 500, 500, 500, 500])
                
    def populate_scatter_plot_ui_elements(self):
        """Populates the UI with elements for creating a scatter plot."""
        self.scatter_x_var_select.clearCheckboxes()
        self.scatter_y_var_select.clearCheckboxes()        
        self.scatter_color_group_select.clearCheckboxes()
        self.scatter_marker_group_select.clearCheckboxes()
        #get the non-group columns and add to x and y variable select
        non_group_cols = data_frame_functions.get_non_group_columns_list(self.selected_sheet_df)
        self.scatter_x_var_select.addCheckBoxes(non_group_cols)     
        self.scatter_y_var_select.addCheckBoxes(non_group_cols)
        #get the group columns and add to color and marker variable select
        group_cols = data_frame_functions.get_group_columns(self.selected_sheet_df)
        self.scatter_color_group_select.addCheckBoxes(group_cols)
        self.scatter_marker_group_select.addCheckBoxes(group_cols)
        #construct dictionary for group filtering
        self.group_filtering_dict = data_frame_functions.get_group_filtering_dict(self.selected_sheet_df)
        
    def populate_bar_plot_ui_elements(self):
        """Clears and then populates the UI with elements for creating a bar plot."""
        self.bar_group_select.clearCheckboxes()
        self.bar_group_filtering_select.clearCheckboxes()
        self.bar_sub_group_select.clearCheckboxes()
        self.bar_y_var_select.clearCheckboxes()
        #add group columns to group select list
        group_cols = data_frame_functions.get_group_columns(self.selected_sheet_df)
        self.bar_group_select.addGroupSelectCheckBoxes(group_cols)  
        #add group columns to sub group select list
        self.bar_sub_group_select.addCheckBoxes(group_cols)
        #add non-group columns to add to y-var select list
        non_group_cols = data_frame_functions.get_non_group_columns_list(self.selected_sheet_df)
        self.bar_y_var_select.addCheckBoxes(non_group_cols)        

    def populate_box_plot_ui_elements(self):
        """Clears and then populates the UI with elements for creating a box plot."""
        self.box_group_select.clearCheckboxes()
        self.box_group_filtering_select.clearCheckboxes()
        self.box_y_var_select.clearCheckboxes()
        #add group columns to group select list
        group_cols = data_frame_functions.get_group_columns(self.selected_sheet_df)
        self.box_group_select.addGroupSelectCheckBoxes(group_cols)        
        #add non-group columns to add to y-var select list
        non_group_cols = data_frame_functions.get_non_group_columns_list(self.selected_sheet_df)
        self.box_y_var_select.addCheckBoxes(non_group_cols)
        #add group columns to color select list
        self.box_color_group_select.addCheckBoxes(group_cols)

    
    def get_scatter_info(self, figure_dict, figure, data_frame):
        """Gets the ttest info for a scatter plot as well as info about the trendline if one is currently displayed on the graph.
           :param: figure_dict - The dictionary containing information about the figure.
           :param: figure - The figure object.
           :param: data_frame - The data frame used to create this figure.
           :return: Returns a string containing the ttest and trendline info if applicable.
        """
        t,p = sp.ttest_ind(data_frame[figure_dict["x"]], data_frame[figure_dict["y"]], equal_var=True, nan_policy="omit")
        standard_ttest_str = "T-Value: " + str(round(t, 5)) + "  P-Value: " + str(round(p, 5))
        if figure_dict.get("trendline") == "ols":
            fit_results = px.get_trendline_results(figure).px_fit_results.iloc[0]
            rsquared = str(round(fit_results.rsquared, 5))
            b = str(round(fit_results.params[0], 5))
            m = str(round(fit_results.params[1], 5))
            equation_str = "Trendline Equation: y = " + m + " * x + " + b + "   R^2 = " + rsquared
            return standard_ttest_str + "     " + equation_str
        return standard_ttest_str

    def display_svg(self, file_path:str, width, height):
        """Loads an svg image of a graph and displays it in the UI.
           :param: file_path - The file path as a string for the svg image to be loaded.
           :param: width - The width in pixels of the image.
           :param: height - The height in pixels of the image.
        """
        self.svg_display.load(file_path)
        if width > 500 or height > 500:
            if width > height:
                scale_factor = 500 / width
            else:
                scale_factor = 500 / height
        else:
            scale_factor = 1
        self.svg_display.setFixedWidth(scale_factor * width)
        self.svg_display.setFixedHeight(scale_factor * height)
