from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
import pickle
from GraphKey.app.modules import path_globals
from pathlib import Path


class EditGraphFrame(QScrollArea):
    """QScrollarea that contains options for editing an existing graph"""
    
    changes_saved = pyqtSignal()

    figure_elements = [" ", "Title","X-Axis Title", "Y-Axis Title", "X-Axis Labels", "Y-Axis Labels", "Legend Title", "Legend Elements"]
    font_families = ["Arial", "Calibri", "Times New Roman", "Verdana"]

    def __init__(self, parent):
        """Adds the window elements to the Edit Graph frame."""
        super().__init__(parent)
        #create widget and layout that will hold graph editing options
        self.widget = QWidget()
        self.vbox = QVBoxLayout()
        self.widget.setLayout(self.vbox)
        self.setWidgetResizable(True)
        self.setWidget(self.widget)
        self.vbox.setAlignment(Qt.AlignTop)
        #TITLE LABEL AND ENTRY BOX
        self.vbox.addWidget(QLabel("Graph Title"))
        self.graph_title_entry_box = QLineEdit()
        self.vbox.addWidget(self.graph_title_entry_box)
        #SIZE LABEL AND ENTRY BOXES
        self.vbox.addWidget(QLabel("Size in Inches"))        
        size_elements_frame = QFrame()        
        size_elements_frame.setFrameShape(QFrame.NoFrame)
        size_elements_frame.setLineWidth(0)
        size_elements_frame.setContentsMargins(0,0,0,0)
        size_elements_frame.setFrameShape(QFrame.NoFrame)
        size_elements_frame_layout = QHBoxLayout()
        size_elements_frame_layout.setContentsMargins(0, 0, 0, 0)
        size_elements_frame.setLayout(size_elements_frame_layout)
        self.width_entry_box = QLineEdit()
        self.width_entry_box.setValidator(QDoubleValidator(0.0,20.0,10))
        self.height_entry_box = QLineEdit()
        self.height_entry_box.setValidator(QDoubleValidator(0.0,20.0,10))
        size_elements_frame_layout.addWidget(self.width_entry_box)
        size_elements_frame_layout.addWidget(QLabel("by"))
        size_elements_frame_layout.addWidget(self.height_entry_box)
        self.vbox.addWidget(size_elements_frame)
        #X-AXIS LABEL AND ENTRY BOX
        self.vbox.addWidget(QLabel("X-Axis Title"))
        self.x_axis_title_entry_box = QLineEdit()
        self.vbox.addWidget(self.x_axis_title_entry_box)
        #X-AXIS RANGE
        x_axis_range_frame = QFrame()
        x_axis_range_frame_layout = QHBoxLayout()
        x_axis_range_frame.setLayout(x_axis_range_frame_layout)
        self.x_range_min = QLineEdit()
        self.x_range_min.setValidator(QDoubleValidator())
        self.x_range_max = QLineEdit()
        self.x_range_max.setValidator(QDoubleValidator())
        x_axis_range_frame_layout.addWidget(QLabel("X Min"))
        x_axis_range_frame_layout.addWidget(self.x_range_min)
        x_axis_range_frame_layout.addWidget(QLabel("X Max"))
        x_axis_range_frame_layout.addWidget(self.x_range_max)
        self.vbox.addWidget(x_axis_range_frame)
        #Y-AXIS LABEL AND ENTRY BOX
        self.vbox.addWidget(QLabel("Y-Axis Title"))
        self.y_axis_title_entry_box = QLineEdit()
        self.vbox.addWidget(self.y_axis_title_entry_box)
        #Y-AXIS RANGE
        y_axis_range_frame = QFrame()
        y_axis_range_frame_layout = QHBoxLayout()
        y_axis_range_frame.setLayout(y_axis_range_frame_layout)
        self.y_range_min = QLineEdit()
        self.y_range_min.setValidator(QDoubleValidator())
        self.y_range_max = QLineEdit()
        self.y_range_max.setValidator(QDoubleValidator())
        y_axis_range_frame_layout.addWidget(QLabel("Y Min"))
        y_axis_range_frame_layout.addWidget(self.y_range_min)
        y_axis_range_frame_layout.addWidget(QLabel("Y Max"))
        y_axis_range_frame_layout.addWidget(self.y_range_max)
        self.vbox.addWidget(y_axis_range_frame)
        #ENABLE LOG X-AXIS CHECKBOX
        self.enable_log_x_axis_checkbox = QCheckBox("Enable logarithmic x-axis")
        self.vbox.addWidget(self.enable_log_x_axis_checkbox)
        #ENABLE LOG Y-AXIS CHECKBOX
        self.enable_log_y_axis_checkbox = QCheckBox("Enable logarithmic y-axis")
        self.vbox.addWidget(self.enable_log_y_axis_checkbox)
        #ENABLE ERROR BARS BAR GRAPH OPTION ONLY
        self.enable_error_bars_checkbox = QCheckBox("Enable error bars")
        self.enable_error_bars_checkbox.hide()
        self.vbox.addWidget(self.enable_error_bars_checkbox)
        #ENABLE TRENDLINE SCATTER GRAPH OPTION ONLY
        self.enable_trendline_checkbox = QCheckBox("Enable linear trendline")
        self.enable_trendline_checkbox.hide()
        self.vbox.addWidget(self.enable_trendline_checkbox)      
        #COLOR/MARKER DROPDOWNS
        color_and_marker_dropdown_layout = QHBoxLayout()
        color_and_marker_dropdown_layout.setContentsMargins(0, 0, 0, 0)
        color_and_marker_dropdown_widget = QWidget()       
        #COLORS DROPDOWN
        self.color_dropdown_widget = QWidget()
        color_dropdown_layout = QVBoxLayout()
        color_dropdown_layout.setContentsMargins(0, 0, 0, 0)
        configs = pickle.load(open(Path(path_globals.settings_folder_path / "save_color_configs.p"), "rb"))
        self.saved_color_themes = configs["saved_themes"]
        self.color_dropdown = QComboBox()
        for theme in self.saved_color_themes:
            self.color_dropdown.addItem(theme["name"])
        color_dropdown_layout.addWidget(QLabel("Marker Color"))
        color_dropdown_layout.addWidget(self.color_dropdown)
        self.color_dropdown_widget.setLayout(color_dropdown_layout)
        color_and_marker_dropdown_layout.addWidget(self.color_dropdown_widget)
        #SYMBOLS DROPDOWN
        self.symbol_dropdown_widget = QWidget()
        symbol_dropdown_layout = QVBoxLayout()
        symbol_dropdown_layout.setContentsMargins(0, 0, 0, 0)
        configs = pickle.load(open(Path(path_globals.settings_folder_path / "save_shape_configs.p"), "rb"))
        self.saved_shape_themes = configs["saved_themes"]
        self.symbol_dropdown = QComboBox()
        for theme in self.saved_shape_themes:
            self.symbol_dropdown.addItem(theme["name"])
        symbol_dropdown_layout.addWidget(QLabel("Marker Symbol"))
        symbol_dropdown_layout.addWidget(self.symbol_dropdown)
        self.symbol_dropdown_widget.setLayout(symbol_dropdown_layout)
        color_and_marker_dropdown_layout.addWidget(self.symbol_dropdown_widget)
        color_and_marker_dropdown_widget.setLayout(color_and_marker_dropdown_layout)
        self.vbox.addWidget(color_and_marker_dropdown_widget)
        #FONTS FRAME
        fonts_frame = QFrame()
        fonts_frame_layout = QHBoxLayout()
        fonts_frame_layout.setContentsMargins(0,0,0,0)
        fonts_frame.setLayout(fonts_frame_layout)
        self.vbox.addWidget(fonts_frame)
        #FONT ELEMENT SELECT
        font_element_frame = QFrame()
        font_element_frame_layout = QVBoxLayout()
        font_element_frame_layout.setContentsMargins(0,0,0,0)
        font_element_frame.setLayout(font_element_frame_layout)
        fonts_frame_layout.addWidget(font_element_frame)
        font_element_frame_layout.addWidget(QLabel("Text Element"))
        self.font_element_dropdown = QComboBox()
        self.font_element_dropdown.addItems(self.figure_elements)
        font_element_frame_layout.addWidget(self.font_element_dropdown)
        #FONT SIZE SELECT
        font_size_frame = QFrame()
        font_size_frame_layout = QVBoxLayout()
        font_size_frame_layout.setContentsMargins(0,0,0,0)
        font_size_frame.setLayout(font_size_frame_layout)
        fonts_frame_layout.addWidget(font_size_frame)
        font_size_frame_layout.addWidget(QLabel("Font Size"))
        self.font_size_entry_box = QLineEdit()
        self.font_size_entry_box.setValidator(QIntValidator(0, 100))
        font_size_frame_layout.addWidget(self.font_size_entry_box)
        #FONT FAMILY SELECT
        font_family_frame = QFrame()
        font_family_frame_layout = QVBoxLayout()
        font_family_frame_layout.setContentsMargins(0,0,0,0)
        font_family_frame.setLayout(font_family_frame_layout)
        fonts_frame_layout.addWidget(font_family_frame)
        font_family_frame_layout.addWidget(QLabel("Font Family"))
        self.font_family_dropdown = QComboBox()
        self.font_family_dropdown.addItems(self.font_families)
        font_family_frame_layout.addWidget(self.font_family_dropdown)

        #SAVE CHANGES BUTTON
        self.save_changes_button = QPushButton("Save Changes")
        self.vbox.addStretch(1)
        self.vbox.addWidget(self.save_changes_button)
        self.save_changes_button.clicked.connect(self.save_changes)
        


    def populate_dict_data(self, figure_dict:dict):
        """Populates the current graph setting in the Edit Graph window.
           :param: figure_dict - The dictionary of settings.
        """
        self.figure_dict = figure_dict
        if figure_dict.get("title") != None:
            self.graph_title_entry_box.setText(figure_dict["title"])
        else:
            self.graph_title_entry_box.setText("")

        self.width_entry_box.setText(str(figure_dict["width_inches"]))
        self.height_entry_box.setText(str(figure_dict["height_inches"]))

        if figure_dict.get("x_min") != None: 
            self.x_range_min.setText(str(figure_dict["x_min"]))
        else:
            self.x_range_min.setText("")
        if figure_dict.get("x_max") != None:
            self.x_range_max.setText(str(figure_dict["x_max"]))
        else:
            self.x_range_max.setText("")
        if figure_dict.get("y_min") != None: 
            self.y_range_min.setText(str(figure_dict["y_min"]))
        else:
            self.y_range_min.setText("")
        if figure_dict.get("y_max") != None: 
            self.y_range_max.setText(str(figure_dict["y_max"]))
        else:
            self.y_range_max.setText("")
        labels_dict = figure_dict["labels"]
        if labels_dict.get(figure_dict["x"]) != None:
            self.x_axis_title_entry_box.setText(labels_dict[figure_dict["x"]])
        else:
            self.x_axis_title_entry_box.setText(figure_dict["x"])
        if labels_dict.get(figure_dict["y"]) != None:
            self.y_axis_title_entry_box.setText(labels_dict[figure_dict["y"]])
        else:
            self.y_axis_title_entry_box.setText(figure_dict["y"])
        
        self.enable_log_x_axis_checkbox.setChecked(figure_dict["log_x"])
        self.enable_log_y_axis_checkbox.setChecked(figure_dict["log_y"])

        #if the enable_error_bar_checkbox is shown, fetch its value
        if self.enable_error_bars_checkbox.isVisible():
            self.enable_error_bars_checkbox.setChecked(figure_dict["enable_error_bars"])
        configs = pickle.load(open(Path(path_globals.settings_folder_path / "save_color_configs.p"), "rb"))
        self.current_color_theme = configs["current_theme"]
        index = self.color_dropdown.findText(self.current_color_theme["name"])
        if index >= 0:
            self.color_dropdown.setCurrentIndex(index)        
        if self.symbol_dropdown_widget.isVisible():
            configs = pickle.load(open(Path(path_globals.settings_folder_path / "save_shape_configs.p"), "rb"))
            self.current_symbol_theme = configs["current_theme"]
            index = self.symbol_dropdown.findText(self.current_symbol_theme["name"])
            if index >= 0:
                self.symbol_dropdown.setCurrentIndex(index)        

        #if the enable_trendline_box is shown, determine whether or not it should be checked
        if self.enable_trendline_checkbox.isVisible():
            if figure_dict.get("trendline") != None:
                self.enable_trendline_checkbox.setChecked(True)    


    def save_changes(self):
        """Collects all of the user's selected settings and puts it in a dictionary."""
        try:
            self.figure_dict["title"] = self.graph_title_entry_box.text()
            self.figure_dict["width_inches"] = float(self.width_entry_box.text())
            self.figure_dict["height_inches"] = float(self.height_entry_box.text())
            if len(self.x_range_min.text()) > 0:
                self.figure_dict["x_min"] = float(self.x_range_min.text())
            else:
                self.figure_dict["x_min"] = None
            if len(self.x_range_max.text()) > 0:
                self.figure_dict["x_max"] = float(self.x_range_max.text())
            else:
                self.figure_dict["x_max"] = None
            if len(self.y_range_min.text()) > 0:
                self.figure_dict["y_min"] = float(self.y_range_min.text())
            else:
                self.figure_dict["y_min"] = None
            if len(self.y_range_max.text()) > 0:
                self.figure_dict["y_max"] = float(self.y_range_max.text())
            else:
                self.figure_dict["y_max"] = None
            labels_dict = self.figure_dict["labels"]
            labels_dict[self.figure_dict["x"]] = self.x_axis_title_entry_box.text()   
            labels_dict[self.figure_dict["y"]] = self.y_axis_title_entry_box.text()    
            self.figure_dict["log_x"] = self.enable_log_x_axis_checkbox.isChecked()
            self.figure_dict["log_y"] = self.enable_log_y_axis_checkbox.isChecked()
            #save error bar enables if the UI element is visible
            if self.enable_error_bars_checkbox.isVisible():
                self.figure_dict["enable_error_bars"] = self.enable_error_bars_checkbox.isChecked()      
            if self.enable_trendline_checkbox.isVisible():
                if self.enable_trendline_checkbox.isChecked():
                    self.figure_dict["trendline"] = "ols"
                    self.figure_dict["trendline_color_override"] = "Black"
                else:
                    if self.figure_dict.get("trendline") != None:
                        self.figure_dict.pop("trendline")
            #save selected color configs
            configs = pickle.load(open(Path(path_globals.settings_folder_path / "save_color_configs.p"), "rb"))
            self.saved_color_themes = configs["saved_themes"]
            for theme in self.saved_color_themes:
                if self.color_dropdown.currentText() == theme["name"]:
                    self.figure_dict["color_discrete_sequence"] = theme["colors"]
            if self.symbol_dropdown_widget.isVisible():
                configs = pickle.load(open(Path(path_globals.settings_folder_path / "save_shape_configs.p"), "rb"))
                self.saved_shape_themes = configs["saved_themes"]
                for theme in self.saved_shape_themes:
                    if self.symbol_dropdown.currentText() == theme["name"]:
                        self.figure_dict["symbol_sequence"] = theme["shapes"]
            #save selected font configs
            if self.font_element_dropdown.currentText() != " ":
                font_element = self.font_element_dropdown.currentText()
                self.figure_dict[font_element + " font family"] = self.font_family_dropdown.currentText()
                if self.font_size_entry_box.text() != "":
                    self.figure_dict[font_element + " font size"] = 4/3 * float(self.font_size_entry_box.text())
            self.changes_saved.emit()
        except AttributeError:
            self.show_alert("Must have a graph selected to edit a graph's information...")
        

    def show_bar_graph_options(self):
        """Shows the bar graph options."""
        self.enable_error_bars_checkbox.show()

    def hide_bar_graph_options(self):
        """Hides the bar graph options."""
        self.enable_error_bars_checkbox.hide()

    def show_scatter_graph_options(self):
        """Shows the scatter graph options."""
        self.enable_trendline_checkbox.show()
        self.symbol_dropdown_widget.show()

    def hide_scatter_graph_options(self):
        """Hides the scatter graph options."""
        self.enable_trendline_checkbox.hide()
        self.symbol_dropdown_widget.hide()

    def show_alert(self, text):
        """Shows an alert box.
           :param: text - The text to be displayed in the alert box.
        """
        alert = QMessageBox()
        alert.setWindowTitle("Alert")
        alert.setText(text)
        alert.setStandardButtons(QMessageBox.Ok)
        alert.exec_()

