import sys
import os
import pickle
from PyQt5.QtWidgets import*
from PyQt5.QtCore import *
from PyQt5.Qt import QStandardItemModel, QStandardItem
from PyQt5.QtGui import *
import GraphKey.app.modules.data_export.google_drive_interface.google_drive_functions as gd
from GraphKey.app.modules.data_export.google_drive_interface.google_drive_select_account import *
from GraphKey.app.modules.data_export.google_drive_interface.google_drive_select_folder import *
from GraphKey.app.modules.data_export.google_drive_interface.google_drive_select_file import *
from GraphKey.app.modules import path_globals
from pathlib import Path

class GoogleDriveExportScreen(QWidget):
    """A QWidget class defining the Google Drive Export Screen. This screen shows the user the currently selected Google Drive account and gives them the option to either switch accounts or continue to export files.
    """

    def __init__(self, app, list_of_file_paths, graph_page):
        """Initializes the Google Drive Export Screen.
           :param: app - Variable pointing to the parent application, used for processing events.
           :param: list_of_file_paths - List of files user can choose to export from.
           :param: graph_page - Variable linking to the graph_creation_page instance.
        """
        super().__init__()
        self.widget_app = app
        self.graph_creation_page = graph_page
        exists = os.path.exists(Path(path_globals.settings_folder_path / "google_drive_configs.p"))
        if (not exists):
            configs = {'saved_accounts': [], 'current_account': {'name': 'None', 'account': None}}
            pickle.dump(configs, open(Path(path_globals.settings_folder_path / "google_drive_configs.p"), "w+b"))
        configs = pickle.load(open(Path(path_globals.settings_folder_path / "google_drive_configs.p"), "rb"))
        self.current_account = configs["current_account"]
        self.saved_accounts = configs["saved_accounts"]
        self.file_list = list_of_file_paths
        self.make_window()

    def make_window(self):
        """Creates the Export window and all of its elements."""
        
        layout = QVBoxLayout()
        drive_layout = QVBoxLayout()

        #CURRENT ACCOUNT AREA
        current_account_selected = QLabel("Current Account Selected: {}".format(self.current_account["name"]))


        #BUTTON AREA
        button_widget = QWidget()
        button_layout = QHBoxLayout()
        cancel = QPushButton("Cancel")
        cancel.clicked.connect(lambda: self.cancel())
        switch = QPushButton("Switch Account")
        switch.clicked.connect(lambda: self.switch())
        export = QPushButton("Select Files to Export")
        export.clicked.connect(lambda: self.export())

        button_layout.addWidget(export)
        button_layout.addWidget(switch)
        button_layout.addWidget(cancel)
        button_widget.setLayout(button_layout)

        #WHOLE WIDGET
        drive_layout.addWidget(current_account_selected)
        drive_layout.addWidget(button_widget)
        drive_widget = QWidget()
        drive_widget.setLayout(drive_layout)

        layout.addWidget(drive_widget)
        self.setLayout(layout)
        self.setWindowTitle("Export to Google Drive")
        self.show()

    def switch(self):
        """If the user decides to switch accounts, this function will create the Google Drive Account Settings window."""
        settings = GoogleDriveAccountSettings(self.widget_app, self.graph_creation_page)
        settings.exec()
        self.close()
        export = GoogleDriveExportScreen(self.widget_app, self.file_list, self.graph_creation_page)
        export.show()

    def export(self):
        """If the user decides to export files, this functions will create the Google Drive Select File window."""
        if self.current_account["name"] == "None":
            msg = QMessageBox()
            msg.setText("No account is currently selected, please first switch accounts.")
            msg.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
            msg.setWindowTitle("No Account Selected")
            msg.exec()          
            return
        list_of_file_paths = self.file_list 
        files = GoogleDriveSelectFile(self.widget_app, list_of_file_paths, self.graph_creation_page)
        files.show()
        self.close()

    def cancel(self):
        """If the user decides to cancel, this function will close the window."""
        self.close()


