import pandas as pd
import os

from .utils import *


class DataImport:
    """This class is for data importation."""
    file_path = ''

    def __init__(self):
        """Initializes the data import object."""
        pass

    def import_pickled_dfs_from_folder(self, folder_path:str):
        """Returns a dictionary with all containing all the dataframes read from folder file_path"""
        df_dict = dict()
        for file in os.scandir(folder_path):
            df_dict[file.name] = pd.read_pickle(file.path)
            df_dict[file.name] = df_dict[file.name].applymap(lambda x: x.strip() if type(x)==str else x)
        return df_dict

    def import_data(self, file_path: str):
        """
        The master method for importing a file.
        Will test to see if the extension is supported and calls the appropriate importing method.
        :param file_path: A file path to the data file to import.
        :returns A dictionary of panda's DataFrames
        """
        # Error checking
        try:
            if file_path is None:
                raise TypeError('File path is Null')
            if not os.path.exists(file_path):
                raise FileNotFoundError('File path does not reach a valid file')

            self.file_path = file_path
            extension = get_extension(file_path)
            if extension == '.csv':
                return self._import_csv()
            elif extension == '.xlsx':
                return self._import_xlsx()
            else:
                raise TypeError('File type not supported')
        except Exception:
            self.show_alert("File Import failed. Please make sure the file type is supported and the file has not already been imported (use Import Revision in this case.")

    def _import_csv(self):
        """
        Handles importing a data file with the .csv extension.
        :return: A dictionary of one panda DataFrame (since a CSV file has only one "sheet")
        """
        df = pd.read_csv(self.file_path, dtype=object)
        return {get_filename(self.file_path): df}

    def _import_xlsx(self):
        """
        Handles importing from a data file with the .xlsx (EXCEL) file
        :return: A dictionary of panda DataFrames (one per sheet)
        """
        return pd.read_excel(self.file_path, None)

    def show_alert(self, text):
        """
        Shows alert message if something went wrong.
        :param text: The message to display.
        """
        alert = QMessageBox()
        alert.setWindowTitle("Alert")
        alert.setText(text)
        alert.setStandardButtons(QMessageBox.Ok)
        alert.exec_()

