import enum

import pandas

from GraphKey.app.modules.graphing.figure.box_plot import BoxPlot
from GraphKey.app.modules.graphing.figure.bar_plot import BarPlot
from GraphKey.app.modules.graphing.figure.scatter_plot import ScatterPlot


class SupportedFigures(enum.Enum):
    Box = 0,
    Scatter = 1,
    Bar = 2


class FigureFactory:
    @staticmethod
    def factory(df: pandas.DataFrame, figure_type: SupportedFigures):
        """
        This method generates the Figure object desired based on the figure type provided. This will allow the GUI
        to not have to be rewritten every time a new graph is added to the app, for all it cares about is having a
        Figure object. This will be the only file that needs to be changed to add new functionality.
        :param df: A Pandas DataFrame required by the Figure class
        :param figure_type: A string denoting which figure to select (eg 'scatter', 'dot', 'box')
        :return: A Figure object
        :raises: TypeError if the figure type is not found or supported
        """
        if figure_type is SupportedFigures.Box:
            return BoxPlot(df)
        if figure_type is SupportedFigures.Scatter:
            return ScatterPlot(df)
        if figure_type is SupportedFigures.Bar:
            return BarPlot(df)
        raise TypeError(f'Type {figure_type} is not supported')
