import sys
import os
import pickle
from plotly.validators.scatter.marker import SymbolValidator
from PyQt5.QtWidgets import*
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from functools import partial
from GraphKey.app.modules import path_globals
from pathlib import Path
from GraphKey.app.modules.graphing.settings.edit_graph_frame_lite import *

class Settings(QWidget):
    """This class is for the Default Graph Preferences screen.""" 
    def __init__(self, tab_index): #, configs):
        """Initializes the Default Graph Preferences screen.
           :param: tab_index - The current tab the user is on.
        """
        super().__init__()
        self.figure_elements = ["Title","X-Axis Title", "Y-Axis Title", "X-Axis Labels", "Y-Axis Labels", "Legend Title", "Legend Elements"]
        exists = os.path.exists(Path(path_globals.settings_folder_path / "save_color_configs.p"))
        if (not exists):
            configs = {'saved_themes': [{'name': 'Earthy Tones', 'colors': ['#aa5500', '#00aa7f', '#55aa00', '#ffaa00', '#946300']}, {'name': 'Easter Colors', 'colors': ['#ffaaff', '#55ffff', '#ffff7f', '#00ff00', '#ffffff']}, {'name': 'Rainbow', 'colors': ['#ff0000', '#ffaa00', '#ffff00', '#00aa00', '#0055ff', '#aa55ff', '#ff55ff', '#000000', '#ffffff']}, {'name': 'Basic', 'colors': ['#007c00', '#f10000', '#0055ff', '#f7793a']}], 'current_theme': {'name': 'Rainbow', 'colors': ['#ff0000', '#ffaa00', '#ffff00', '#00aa00', '#0055ff', '#aa55ff', '#ff55ff', '#000000', '#ffffff']}}
            pickle.dump(configs, open(Path(path_globals.settings_folder_path / "save_color_configs.p"), "w+b"))
        configs = pickle.load(open(Path(path_globals.settings_folder_path / "save_color_configs.p"), "rb"))
        self.color_theme = configs["current_theme"] 
        self.saved_color_themes = configs["saved_themes"]
        exists = os.path.exists(Path(path_globals.settings_folder_path / "save_shape_configs.p"))
        if (not exists):
            configs = {'saved_themes': [{'name': 'Boxy', 'shapes': ['square', 'cross', 'x', 'pentagon']}, {'name': 'Basic', 'shapes': ['square', 'triangle-up', 'star', 'circle']}, {'name': 'Default', 'shapes': ['circle', 'star', 'triangle-up', 'cross', 'diamond', 'pentagon']}], 'current_theme': {'name': 'Default', 'shapes': ['circle', 'star', 'triangle-up', 'cross', 'diamond', 'pentagon']}}
            pickle.dump(configs, open(Path(path_globals.settings_folder_path / "save_shape_configs.p"), "w+b"))
        configs = pickle.load(open(Path(path_globals.settings_folder_path / "save_shape_configs.p"), "rb"))
        self.shape_theme = configs["current_theme"]
        self.saved_shape_themes = configs["saved_themes"]
        exists = os.path.exists(Path(path_globals.settings_folder_path / "save_other_configs.p"))
        if (not exists):
            temp_dict = {}
            temp_dict["width_inches"] = 5
            temp_dict["height_inches"] = 5
            temp_dict["log_x"] = False
            temp_dict["log_y"] = False
            temp_dict["enable_error_bars"] = False           
            for title in self.figure_elements:
                temp_dict[title + " font family"] = "Arial"
                temp_dict[title + " font size"] = float(10)
            pickle.dump(temp_dict, open(Path(path_globals.settings_folder_path / "save_other_configs.p"), "w+b"))
        configs = pickle.load(open(Path(path_globals.settings_folder_path / "save_other_configs.p"), "rb"))
        self.figure_dict = configs
        self.current_tab_index = tab_index
        self.make_window()

    def make_window(self):
        """Creates and adds all the window elements."""
        self.setWindowTitle('Graph Settings')
        tabs = QTabWidget()

        ####COLOR TAB####
        color_tab_layout = QHBoxLayout()
        color_layout = QVBoxLayout()

        #CURRENT THEME AREA
        current_color_theme = QLabel("Current Theme Selected: {}".format(self.color_theme["name"]))
        current_palette = PaletteColorHorizontal(self.color_theme["colors"])

        #SAVED THEMES AREA
        radio_button_list = []
        scroll_widget = QWidget()
        scroll_layout = QVBoxLayout()
        different_theme = QLabel("Saved Themes")
        i = 0
        for t in self.saved_color_themes:
            radio_button_list.append(QRadioButton("{}".format(t["name"])))
            palette = PaletteColorHorizontal(t["colors"])
            scroll_layout.addWidget(radio_button_list[i])
            scroll_layout.addWidget(palette)
            i = i+1
        scroll_widget.setLayout(scroll_layout)

        scroll_theme = QScrollArea()
        scroll_theme.setWidget(scroll_widget)
        scroll_theme.setWidgetResizable(True)

        #BUTTON AREA
        button_widget = QWidget()
        button_layout = QHBoxLayout()
        apply_changes = QPushButton("Apply Changes")
        apply_changes.clicked.connect(lambda: self.apply_changes())
        create_theme = QPushButton("Create Theme")
        create_theme.clicked.connect(lambda: self.create_color_theme(tabs))
        select = QPushButton("Select Theme")
        select.clicked.connect(lambda: self.select_color_theme(radio_button_list, tabs))
        cancel = QPushButton("Cancel")
        cancel.clicked.connect(lambda: self.cancel())
        delete_theme = QPushButton("Delete Theme")
        delete_theme.clicked.connect(lambda: self.delete_color_theme(radio_button_list, tabs))
        
        button_layout.addWidget(select)
        button_layout.addWidget(create_theme)
        button_layout.addWidget(delete_theme)
        button_layout.addWidget(apply_changes)
        button_layout.addWidget(cancel)
        button_widget.setLayout(button_layout)

        #COLOR AREA - COMPRISED OF CURRENT THEME, SAVED THEMES, AND BUTTON AREAS
        color_layout.addWidget(current_color_theme)
        color_layout.addWidget(current_palette)
        color_layout.addWidget(different_theme)
        color_layout.addWidget(scroll_theme)
        color_layout.addWidget(button_widget)
        color_widget = QWidget()
        color_widget.setLayout(color_layout)

        #WHOLE SCREEN
        color_tab_layout.addWidget(color_widget)
        color_tab = QWidget()
        color_tab.setLayout(color_tab_layout)
        


        ####SHAPE TAB####
        shape_tab_layout = QHBoxLayout()
        shape_layout = QVBoxLayout()

        #CURRENT THEME AREA
        current_shape_theme = QLabel("Current Theme Selected: {}".format(self.shape_theme["name"]))
        current_palette = PaletteIconHorizontal(self.shape_theme["shapes"])

        #SAVED THEMES AREA
        radio_button_list_icons = []
        scroll_widget = QWidget()
        scroll_layout = QVBoxLayout()
        different_theme = QLabel("Saved Themes")
        i = 0
        for t in self.saved_shape_themes:
            radio_button_list_icons.append(QRadioButton("{}".format(t["name"])))
            palette = PaletteIconHorizontal(t["shapes"])
            scroll_layout.addWidget(radio_button_list_icons[i])
            scroll_layout.addWidget(palette)
            i = i+1
        scroll_widget.setLayout(scroll_layout)

        scroll_theme = QScrollArea()
        scroll_theme.setWidget(scroll_widget)
        scroll_theme.setWidgetResizable(True)

        #BUTTON AREA
        button_widget = QWidget()
        button_layout = QHBoxLayout()
        apply_changes = QPushButton("Apply Changes")
        apply_changes.clicked.connect(lambda: self.apply_changes())
        create_theme = QPushButton("Create Theme")
        create_theme.clicked.connect(lambda: self.create_shape_theme(tabs))
        select = QPushButton("Select Theme")
        select.clicked.connect(lambda: self.select_shape_theme(radio_button_list_icons, tabs))
        cancel = QPushButton("Cancel")
        cancel.clicked.connect(lambda: self.cancel())
        delete_theme = QPushButton("Delete Theme")
        delete_theme.clicked.connect(lambda: self.delete_shape_theme(radio_button_list_icons, tabs))
        
        button_layout.addWidget(select)
        button_layout.addWidget(create_theme)
        button_layout.addWidget(delete_theme)
        button_layout.addWidget(apply_changes)
        button_layout.addWidget(cancel)
        button_widget.setLayout(button_layout)

        #COLOR AREA - COMPRISED OF CURRENT THEME, SAVED THEMES, AND BUTTON AREAS
        shape_layout.addWidget(current_shape_theme)
        shape_layout.addWidget(current_palette)
        shape_layout.addWidget(different_theme)
        shape_layout.addWidget(scroll_theme)
        shape_layout.addWidget(button_widget)
        shape_widget = QWidget()
        shape_widget.setLayout(shape_layout)

        #WHOLE SCREEN
        shape_tab_layout.addWidget(shape_widget)
        shape_tab = QWidget()
        shape_tab.setLayout(shape_tab_layout)
		
        ####OTHER TAB####
        other_tab_layout = QVBoxLayout()
        self.other_options = EditGraphFrameLite(self.figure_dict)

        #BUTTON AREA
        button_widget = QWidget()
        button_layout = QHBoxLayout()
        apply_changes = QPushButton("Apply Changes")
        apply_changes.clicked.connect(lambda: self.apply_changes())
        cancel = QPushButton("Cancel")
        cancel.clicked.connect(lambda: self.cancel())
        
        button_layout.addWidget(apply_changes)
        button_layout.addWidget(cancel)
        button_widget.setLayout(button_layout)

        other_tab_layout.addWidget(self.other_options)
        other_tab_layout.addWidget(button_widget)
        other_tab = QWidget()
        other_tab.setLayout(other_tab_layout)

        ####ALL TABS####
        tabs.addTab(color_tab, "Colors")
        tabs.addTab(shape_tab, "Symbols")
        tabs.addTab(other_tab, "Other")
        tabs.setCurrentIndex(self.current_tab_index)
        window_layout = QHBoxLayout()
        window_layout.addWidget(tabs)
        self.setLayout(window_layout)
        self.setUpdatesEnabled(True)

    def apply_changes(self):
        """This functions saves all the graph settings to various pickle files."""
        configs = {"saved_themes": self.saved_color_themes, "current_theme": self.color_theme}
        pickle.dump(configs, open(Path(path_globals.settings_folder_path / "save_color_configs.p"), "wb"))
        configs = {"saved_themes": self.saved_shape_themes, "current_theme": self.shape_theme}
        pickle.dump(configs, open(Path(path_globals.settings_folder_path / "save_shape_configs.p"), "wb"))
        configs = self.other_options.save_changes()
        pickle.dump(configs, open(Path(path_globals.settings_folder_path / "save_other_configs.p"), "wb"))
        self.close()
        return
		
    def select_color_theme(self, buttons, tabs):
        """Once the user selects a theme, this function will update the currently selected theme.
           :param: buttons - The different buttons the user could select.
           :param: tabs - The different tabs on the Default Graph Preferences screen.
        """
        selected = self.get_selected(buttons)
        self.current_tab_index = tabs.currentIndex()
        if (selected is None):
            msg = QMessageBox()
            msg.setText("No theme was selected, failed to select theme.")
            msg.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
            msg.setWindowTitle("Failed to Select Theme")
            msg.exec()
            return
        self.color_theme = self.saved_color_themes[selected]
        self.apply_changes()
        setting = Settings(self.current_tab_index) 
        setting.show()

    def select_shape_theme(self, icons, tabs):
        """Once the user selects a theme, this function will update the currently selected theme.
           :param: icons - The different icons the user could select.
           :param: tabs - The different tabs on the Default Graph Preferences screen.
        """
        selected = self.get_selected(icons)
        self.current_tab_index = tabs.currentIndex()
        if (selected is None):
            msg = QMessageBox()
            msg.setText("No theme was selected, failed to select theme.")
            msg.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
            msg.setWindowTitle("Failed to Select Theme")
            msg.exec()
            return
        self.shape_theme = self.saved_shape_themes[selected]
        self.apply_changes()
        setting = Settings(self.current_tab_index) 
        setting.show()

    def create_color_theme(self, tabs):
        """If the user decides to create a new color theme, this function will open the New Color Theme window.
           :param: tabs - The different tabs on the Default Graph Preferences screen.
        """
        self.current_tab_index = tabs.currentIndex()
        theme = []
        new_theme = New_Color_Theme(theme)
        new_theme.setWindowTitle("Create Theme")
        new_theme.exec()
        if (new_theme.name is None):
            return
        names = [theme_name["name"] for theme_name in self.saved_color_themes]
        name = new_theme.name
        while (name in names):
            name, ok = QInputDialog.getText(self, 'Theme Name', 'Theme name already exists. Enter a new Theme Name:')
            if ok:
                continue
            else:
                return
        if (not new_theme.color_theme):
            msg = QMessageBox()
            msg.setText("No colors were selected, failed to save theme.")
            msg.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
            msg.setWindowTitle("Failed to Save Theme")
            msg.exec()
            return

        self.saved_color_themes.append({"name":name, "colors":new_theme.color_theme})
        self.apply_changes()
        setting = Settings(self.current_tab_index) 
        setting.show()
	
    def create_shape_theme(self, tabs):
        """If the user decides to create a new shape theme, this function will open the New Shape Theme window.
           :param: tabs - The different tabs on the Default Graph Preferences screen.
        """
        self.current_tab_index = tabs.currentIndex()
        theme = []
        new_theme = New_Shape_Theme(theme)
        new_theme.setWindowTitle("Create Theme")
        new_theme.exec()
        if (new_theme.name is None):
            return
        names = [theme_name["name"] for theme_name in self.saved_shape_themes]
        name = new_theme.name
        while (name in names):
            name, ok = QInputDialog.getText(self, 'Theme Name', 'Theme name already exists. Enter a new Theme Name:')
            if ok:
                continue
            else:
                return
        if (not new_theme.shape_theme):
            msg = QMessageBox()
            msg.setText("No symbols were selected, failed to save theme.")
            msg.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
            msg.setWindowTitle("Failed to Save Theme")
            msg.exec()
            return

        self.saved_shape_themes.append({"name":name, "shapes":new_theme.shape_theme})
        self.apply_changes()
        setting = Settings(self.current_tab_index) 
        setting.show()

    def cancel(self):
        """If the user chooses to cancel, this function will close the window."""
        self.close()

    def delete_color_theme(self, buttons, tabs):
        """This function allows the user to delete the selected theme.
           :param: buttons - The different buttons the user could select.
           :param: tabs - The different tabs on the Default Graph Preferences screen.
        """
        selected = self.get_selected(buttons)
        self.current_tab_index = tabs.currentIndex()
        if (selected is None):
            msg = QMessageBox()
            msg.setText("No theme was selected, failed to delete theme.")
            msg.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
            msg.setWindowTitle("Failed to Delete Theme")
            msg.exec()
            return
        if (self.saved_color_themes[selected]["name"] == self.color_theme["name"]):
            msg = QMessageBox()
            msg.setText("You cannot delete the theme that you have chosen for your graph(s). Please choose a different theme for your graph(s), then delete this theme.")
            msg.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
            msg.setWindowTitle("Failed to Delete Theme")
            msg.exec()
            return
        self.saved_color_themes.pop(selected)
        self.apply_changes()
        setting = Settings(self.current_tab_index) 
        setting.show()

    def delete_shape_theme(self, buttons, tabs):
        """This function allows the user to delete the selected theme.
           :param: buttons - The different buttons the user could select.
           :param: tabs - The different tabs on the Default Graph Preferences screen.
        """
        selected = self.get_selected(buttons)
        self.current_tab_index = tabs.currentIndex()
        if (selected is None):
            msg = QMessageBox()
            msg.setText("No theme was selected, failed to delete theme.")
            msg.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
            msg.setWindowTitle("Failed to Delete Theme")
            msg.exec()
            return
        if (self.saved_shape_themes[selected]["name"] == self.shape_theme["name"]):
            msg = QMessageBox()
            msg.setText("You cannot delete the theme that you have chosen for your graph(s). Please choose a different theme for your graph(s), then delete this theme.")
            msg.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
            msg.setWindowTitle("Failed to Delete Theme")
            msg.exec()
            return
        self.saved_shape_themes.pop(selected)
        self.apply_changes()
        setting = Settings(self.current_tab_index) 
        setting.show()

    def get_selected(self, buttons):
        """This functions figures out which button is currently selected.
           :return: The index of the selected button.
        """
        i = 0
        for button in buttons:
            if button.isChecked():
                return i
            i = i+1

			
			
#### SHAPE CLASSES ####
class PaletteIconButton(QToolButton):
    """This class is for generating the shape theme preview buttons."""
    def __init__(self, icon):
        """Initializes the color button."""
        super().__init__()
        icon_path = str(icon) + ".png" 
        filepath = str(Path(path_globals.shapes_folder_path / icon_path))
        self.setIcon(QIcon(filepath))
        self.setStyleSheet("background-color: white")

class PaletteIconHorizontal(QWidget):
    """This class is for generating the shape theme preview palettes."""
    def __init__(self, shapes, *args, **kwargs):
        """Initializes the color palette."""
        super().__init__(*args, **kwargs)
        palette = QHBoxLayout()
        for s in shapes:
            b = PaletteIconButton(s)
            palette.addWidget(b)
        self.setLayout(palette)
		
class ShapeChoices(QDialog):
    """This class creates a window where the user can select a shape to add to their theme."""
    def __init__(self, *args, **kwargs):
        """Initializes the window with all the various shape icons."""
        super().__init__(*args, **kwargs)
		
        self.buttons = []
        self.buttons_names = []
        self.selected_icon = None
		
        palette = QHBoxLayout()
        icons_widget = QWidget()
        layout = QVBoxLayout()
        
        self.square = PaletteIconButton("square")
        self.square.setCheckable(True)
        self.square.setChecked(True) 
        self.buttons.append(self.square) 
        self.buttons_names.append("square")
        self.selected_icon = "square"
        self.square.clicked.connect(lambda: self.only_one_selected(0))	
        palette.addWidget(self.square)
        
        self.triangle = PaletteIconButton("triangle-up")
        self.triangle.setCheckable(True)
        self.triangle.setChecked(False) 
        self.buttons.append(self.triangle) 
        self.buttons_names.append("triangle-up")
        self.triangle.clicked.connect(lambda: self.only_one_selected(1))	
        palette.addWidget(self.triangle)
        
        self.star = PaletteIconButton("star")
        self.star.setCheckable(True)
        self.star.setChecked(False) 
        self.buttons.append(self.star) 
        self.buttons_names.append("star")
        self.star.clicked.connect(lambda: self.only_one_selected(2))	
        palette.addWidget(self.star)
        
        self.diamond = PaletteIconButton("diamond")
        self.diamond.setCheckable(True)
        self.diamond.setChecked(False) 
        self.buttons.append(self.diamond) 
        self.buttons_names.append("diamond")
        self.diamond.clicked.connect(lambda: self.only_one_selected(3))	
        palette.addWidget(self.diamond)
        
        self.cross = PaletteIconButton("cross")
        self.cross.setCheckable(True)
        self.cross.setChecked(False) 
        self.buttons.append(self.cross) 
        self.buttons_names.append("cross")
        self.cross.clicked.connect(lambda: self.only_one_selected(4))	
        palette.addWidget(self.cross)
        
        self.x = PaletteIconButton("x")
        self.x.setCheckable(True)
        self.x.setChecked(False) 
        self.buttons.append(self.x) 
        self.buttons_names.append("x")
        self.x.clicked.connect(lambda: self.only_one_selected(5))	
        palette.addWidget(self.x)
        
        self.circle = PaletteIconButton("circle")
        self.circle.setCheckable(True)
        self.circle.setChecked(False) 
        self.buttons.append(self.circle) 
        self.buttons_names.append("circle")
        self.circle.clicked.connect(lambda: self.only_one_selected(6))	
        palette.addWidget(self.circle)
        
        self.pentagon = PaletteIconButton("pentagon")
        self.pentagon.setCheckable(True)
        self.pentagon.setChecked(False) 
        self.buttons.append(self.pentagon) 
        self.buttons_names.append("pentagon")
        self.pentagon.clicked.connect(lambda: self.only_one_selected(7))	
        palette.addWidget(self.pentagon)
        
        icons_widget.setLayout(palette)	
        layout.addWidget(icons_widget)
        
        select = QPushButton("Select Shape")
        select.clicked.connect(lambda: self.select())
        cancel = QPushButton("Cancel")
        cancel.clicked.connect(lambda: self.cancel())
        
        buttons = QHBoxLayout()
        buttons.addWidget(select)
        buttons.addWidget(cancel)
        buttons_widget = QWidget()
        buttons_widget.setLayout(buttons)
        layout.addWidget(buttons_widget)
        
        self.setLayout(layout)
        
    
    def only_one_selected(self, i):
        """This function ensures that only one shape is selected at a time.
           :param: i - the index of the selected shape.
        """
        for button in self.buttons:
            button.setChecked(False)
        self.buttons[i].setChecked(True)
        self.selected_icon = self.buttons_names[i]
        
    def select(self):
        """Selects and closes the window."""
        self.close()
        
    def cancel(self):
        """Closes the window."""
        self.close()
        self.selected_icon = None
        
		
class New_Shape_Theme(QDialog):
    """This class creates a window where the user can choose to add more shapes to their theme or save their theme."""
    def __init__(self, theme):
        """Initializes the New Shape Theme window."""
        super().__init__()

        layout = QVBoxLayout()
        
        self.shape_theme = theme
        self.name = None

        #BUTTON AREA
        button_widget = QWidget()
        button_layout = QHBoxLayout()
        add_shape = QPushButton("Add a Symbol")
        add_shape.clicked.connect(lambda: self.add_shape())
        cancel = QPushButton("Cancel")
        cancel.clicked.connect(lambda: self.cancel_shapes())
        save_theme = QPushButton("Save Theme")
        save_theme.clicked.connect(lambda: self.save_theme())
        button_layout.addWidget(add_shape)
        button_layout.addWidget(save_theme)
        button_layout.addWidget(cancel)
        button_widget.setLayout(button_layout)
        
        theme_preview = QLabel("Theme Preview")
        
        palette = PaletteIconHorizontal(self.shape_theme)

        layout.addWidget(palette) 
        layout.addWidget(button_widget)

        self.setLayout(layout)

    def add_shape(self):
        """This function allows the user to choose a symbol to add to the theme."""
        shape = ShapeChoices()
        shape.setWindowTitle("Add Symbol")
        shape.exec()
        selected_icon = shape.selected_icon
        if selected_icon is not None:
            self.shape_theme.append(selected_icon)
        self.close()
        new_theme = New_Shape_Theme(self.shape_theme)
        new_theme.exec()
        self.shape_theme = new_theme.shape_theme
        self.name = new_theme.name

    def save_theme(self):
        """This function allows the user to save the new theme."""
        name, ok = QInputDialog.getText(self, 'Theme Name', 'Enter a Theme Name:')
        if ok:
            self.name = name
        self.close()

    def cancel_shapes(self):
        """If a user chooses to cancel, this function will close the window."""
        self.close()
        self.name = None
		

			
#### COLOR CLASSES ####
class PaletteColorButton(QPushButton):
    """This class is for generating the Color Preview buttons."""
    def __init__(self, color):
        """Initializes the Color Preview button."""
        super().__init__()
        self.color = color
        self.setStyleSheet("background-color: %s;" % color)

class PaletteColorHorizontal(QWidget):
    """This class is for generating the Color Preview Palettes."""
    def __init__(self, colors, *args, **kwargs):
        """Initializes the Color Preview Palette."""
        super().__init__(*args, **kwargs)
        palette = QHBoxLayout()
        for c in colors:
            b = PaletteColorButton(c)
            palette.addWidget(b)
        self.setLayout(palette)
			
class New_Color_Theme(QDialog):
    """This class allows the user to create a new Color theme."""
    def __init__(self, theme):
        """Creates the window elements for the New Color Theme window."""
        super().__init__()

        layout = QVBoxLayout()
        
        self.color_theme = theme
        self.name = None

        #BUTTON AREA
        button_widget = QWidget()
        button_layout = QHBoxLayout()
        add_color = QPushButton("Add a Color")
        add_color.clicked.connect(lambda: self.add_color())
        cancel = QPushButton("Cancel")
        cancel.clicked.connect(lambda: self.cancel_colors())
        save_theme = QPushButton("Save Theme")
        save_theme.clicked.connect(lambda: self.save_theme())
        button_layout.addWidget(add_color)
        button_layout.addWidget(save_theme)
        button_layout.addWidget(cancel)
        button_widget.setLayout(button_layout)
        
        theme_preview = QLabel("Theme Preview")
        
        palette = PaletteColorHorizontal(self.color_theme)

        layout.addWidget(palette) 
        layout.addWidget(button_widget)

        self.setLayout(layout)

    def add_color(self):
        """This functions allows the user to select and add a color to their new theme."""
        selected_color = QColorDialog.getColor()
        color = selected_color.name()
        if selected_color.isValid():
            self.color_theme.append(color)
        self.close()
        new_theme = New_Color_Theme(self.color_theme)
        new_theme.exec()
        self.color_theme = new_theme.color_theme
        self.name = new_theme.name

    def save_theme(self):
        """This function allows the user to save their new theme."""
        name, ok = QInputDialog.getText(self, 'Theme Name', 'Enter a Theme Name:')
        if ok:
            self.name = name
        self.close()

    def cancel_colors(self):
        """If the user chooses to cancel, this function will close the window."""
        self.close()
        self.name = None


