from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from PyQt5.QtGui import *
import sys, os
from GraphKey.app.modules.graphing.generate_graph_pickle import *
from GraphKey.app.modules import path_globals

class Templates(QWidget):
    '''
    Class is called when in the graphing window and "Apply Template" is
    clicked from the project dropdown. From this class, the user can choose
    a template and apply the template to multiple graphs in the selected sheet.
    '''

    current_template = None

    def __init__(self, file_paths):
        super().__init__()
        self.file_paths = file_paths
        self.config_UI()

    def config_UI(self):
        self.setWindowTitle("Templates")
        self.resize(800, 500)

        apply_template = QPushButton("Apply Template")
        apply_template.clicked.connect(lambda: self.apply_template())

        self.template_model = QFileSystemModel(self)
        self.template_model.setRootPath(path_globals.templates_folder_path_str)
        self.template_model.setNameFilters(["*.template"])
        self.template_model.setNameFilterDisables(False)
        templates = QTreeView()
        templates.setModel(self.template_model)
        templates.setRootIndex(self.template_model.index(path_globals.templates_folder_path_str))
        templates.setColumnHidden(1, True)
        templates.setColumnHidden(2, True)
        templates.setColumnHidden(3, True)
        templates.setHeaderHidden(True)
        templates.setFixedWidth(200)
        templates.clicked.connect(self.template_clicked)

        # Show graphs in sheet folder
        scroll_widget = QWidget()
        scroll_layout = QVBoxLayout()
        scroll_theme = QScrollArea()
        scroll_theme.setWidget(scroll_widget)
        scroll_theme.setWidgetResizable(True)
        scroll_widget.setLayout(scroll_layout)
        self.check_box_list = []
        self.check_all = QCheckBox("Select All")
        self.check_all.stateChanged.connect(lambda:self.checked_all())
        scroll_layout.addWidget(self.check_all)
        i = 0
        for f in self.file_paths:
            self.check_box_list.append(QCheckBox(os.path.splitext(os.path.basename(self.file_paths[i]))[0]))
            scroll_layout.addWidget(self.check_box_list[i])
            i += 1
        
        # Add template and label to left side
        template_label = QLabel("Choose a template", self)
        template_widget = QWidget()
        template_layout = QVBoxLayout()
        template_widget.setLayout(template_layout)
        template_layout.addWidget(template_label)
        template_layout.addWidget(templates)
        
        # Add graphs and label to right side
        graphs_label = QLabel("Select graphs to apply the template to", self)
        graphs_widget = QWidget()
        graphs_layout = QVBoxLayout()
        graphs_widget.setLayout(graphs_layout)
        graphs_layout.addWidget(graphs_label)
        graphs_layout.addWidget(scroll_theme)

        main_layout = QHBoxLayout()
        main_layout.addWidget(template_widget)
        main_layout.addWidget(graphs_widget)
        main_layout.addWidget(apply_template)
        self.setLayout(main_layout)
        self.setUpdatesEnabled(True)
        self.show()

    def template_clicked(self, index):
        self.current_template = self.template_model.fileInfo(index).absoluteFilePath()

    def apply_template(self):
        '''
        When the "Apply Template" button is clicked, this will check if the "Select All" option is checked.
        If so, it will loop through all file paths and apply the template settings to every one of them.
        If not, it will find all the checked file paths and only apply the template to those that are checked.
        '''
        if self.current_template == None:
            self.show_alert("No template selected")
        else:
            checked_list = []
            if self.check_all.isChecked():
                checked_list = self.file_paths
            else:
                i = 0
                for box in self.check_box_list:
                    if box.isChecked():
                        checked_list.append(self.file_paths[i])
                    i += 1

            for checked in checked_list:
                graph_data = getDataFromPickle(checked)
                template_data = getDataFromPickle(self.current_template)
                
                if graph_data["graph_type"] == "scatter":
                    graph_data["color_discrete_sequence"] = template_data["color_discrete_sequence"]
                    if template_data["symbol_sequence"] != None:
                        graph_data["symbol_sequence"] = template_data["symbol_sequence"]
                    graph_data["width_inches"] = template_data["width"]
                    graph_data["height_inches"] = template_data["height"]
                    graph_data["log_x"] = template_data["log_x"]
                    graph_data["log_y"] = template_data["log_y"]
                    createScatterGraphPickle(checked, graph_data["sheet_selected"], graph_data)

                elif graph_data["graph_type"] == "bar":
                    graph_data["color_discrete_sequence"] = template_data["color_discrete_sequence"]
                    graph_data["width_inches"] = template_data["width"]
                    graph_data["height_inches"] = template_data["height"]
                    graph_data["log_x"] = template_data["log_x"]
                    graph_data["log_y"] = template_data["log_y"]
                    createBarGraphPickle(checked, graph_data["sheet_selected"], graph_data)

                elif graph_data["graph_type"] == "box":
                    graph_data["color_discrete_sequence"] = template_data["color_discrete_sequence"]
                    graph_data["width_inches"] = template_data["width"]
                    graph_data["height_inches"] = template_data["height"]
                    graph_data["log_x"] = template_data["log_x"]
                    graph_data["log_y"] = template_data["log_y"]
                    createBoxGraphPickle(checked, graph_data["sheet_selected"], graph_data)

            self.show_alert("Template applied")

    def checked_all(self):
        '''
        Toggles between selecting all graphs and delselecting all graphs
        '''
        if self.check_all.isChecked():
            for box in self.check_box_list:
                box.setChecked(True)
        else:
            for box in self.check_box_list:
                box.setChecked(False)

    def show_alert(self, text):
            alert = QMessageBox()
            alert.setText(text)
            alert.setStandardButtons(QMessageBox.Ok)
            alert.exec_()
