from PyQt5 import QtWidgets,QtCore

from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

from GraphKey.app.modules.gui import welcome_page
from GraphKey.app.modules.gui.graph_creation_window import GraphCreationWindow
from GraphKey.app.modules.gui.window_elements import create_project,view_templates,project
from GraphKey.app.modules.data_import import import_file
from GraphKey.app.modules.static import help
import pickle
import pathlib
from pathvalidate import sanitize_filepath
import os
from pathlib import Path
from shutil import copyfile
from GraphKey.app.modules.data_import.data_frame_functions import * 
from GraphKey.app.modules.gui.window_elements import background_worker
from GraphKey.app.modules.data_import.import_revision import *
from GraphKey.app.modules.graphing.settings.settings import *
from GraphKey.app.modules.graphing.generate_graph_pickle import *
from GraphKey.app.modules.graphing.templates import *
from GraphKey.app.modules.data_export.export import *
from GraphKey.app.modules import path_globals
from PIL import Image

class MainWindow(QMainWindow):
    """The main window for the application. Pages such as the welcome_page and graph_creation_window are displayed within this window"""
    initial_opening = True

    def __init__(self, app):
        """Create a MainWindow object for application app
           :param: app - The QApplication object this window is for
        """
        super(MainWindow,self).__init__()
        self.setWindowTitle('Microbiology Lab Information Management and Visualization System')
        self.projects = []
        self.app = app
        self.threadpool = QThreadPool()
        self.initUI()

    def initUI(self): 

        #To easily translate the file from QtDesigner to our custom function
        MainWindow = self

        MainWindow.resize(722, 494)    
        if self.initial_opening:
            center_screen = QApplication.desktop().screen().rect().center()
            self.move(center_screen - self.rect().center())
            self.initial_opening = False

        #Just init settings and create configs files
        setting = Settings(0) 
       
        #Welcome Page Functions
        self.welcome_page = welcome_page.WelcomePage()
        self.welcome_page.new_project.connect(self.createProjectForm)
        self.welcome_page.open_project.connect(self.openProject)
        self.welcome_page.imported_project.connect(self.importedProject)
        self.welcome_page.view_templates.connect(self.viewTemplates)

        MainWindow.setCentralWidget(self.welcome_page)

        #Defines Menu Bar
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 722, 22))
        self.menubar.setNativeMenuBar(False)
        self.menubar.setObjectName("menubar")

        #Defines the Project Menu
        self.menuproject = QtWidgets.QMenu(self.menubar)
        self.menuproject.setObjectName("menuProject")
        self.menuproject.setTitle("Project")

        #Defines the File Menu
        self.menufiles = QtWidgets.QMenu(self.menubar)
        self.menufiles.setObjectName("menuFile")
        self.menufiles.setTitle("File")

        #Defines the Help Menu
        self.menuhelp = QtWidgets.QMenu(self.menubar)
        self.menuhelp.setObjectName("menuHelp")
        self.menuhelp.setTitle("Help")

        #Sets the menu bar to the above
        MainWindow.setMenuBar(self.menubar)

        #Add things to the project menu 
        
        #Adds graph creation settings to project box
        self.actionDefaultGraphSettings = QtWidgets.QAction(MainWindow)
        self.actionDefaultGraphSettings.setObjectName("actionDefaultGraphSettings")
        self.actionDefaultGraphSettings.setText("Default Graph Preferences")
        self.menuproject.addAction(self.actionDefaultGraphSettings)
        self.actionDefaultGraphSettings.setEnabled(False)

        # Creates a template
        self.actionCreateTemplate = QtWidgets.QAction(MainWindow)
        self.actionCreateTemplate.setObjectName("actionCreateTemplate")
        self.actionCreateTemplate.setText("Create Template")
        self.menuproject.addAction(self.actionCreateTemplate)
        self.actionCreateTemplate.setEnabled(False)
        
        # Apply template to currently selected graph
        self.actionApplyTemplate = QtWidgets.QAction(MainWindow)
        self.actionApplyTemplate.setObjectName("actionApplyTemplate")
        self.actionApplyTemplate.setText("Apply Template")
        self.menuproject.addAction(self.actionApplyTemplate)
        self.actionApplyTemplate.setEnabled(False)

        #Adds things to the file menu

        #Adds import file to the file menu
        self.actionImportFile = QtWidgets.QAction(MainWindow)
        self.actionImportFile.setObjectName("actionImportFile")
        self.actionImportFile.setText("Import File")
        self.menufiles.addAction(self.actionImportFile)
        self.actionImportFile.setEnabled(False)

        #Adds import revision from workbook to file menu
        self.actionImportRevision = QtWidgets.QAction(MainWindow)
        self.actionImportRevision.setObjectName("actionImportRevision")
        self.actionImportRevision.setText("Import Revision")
        self.menufiles.addAction(self.actionImportRevision)
        self.actionImportRevision.setEnabled(False)

        #Adds export to file box
        self.actionExport = QtWidgets.QAction(MainWindow)
        self.actionExport.setObjectName("actionExport")
        self.actionExport.setText("Export File(s)")
        self.menufiles.addAction(self.actionExport)
        self.actionExport.setEnabled(False)

        #Adds export project to file box
        self.actionProjectExport = QtWidgets.QAction(MainWindow)
        self.actionProjectExport.setObjectName("actionProjectExport")
        self.actionProjectExport.setText("Export Selected Project")
        self.menufiles.addAction(self.actionProjectExport)
        self.actionProjectExport.setEnabled(False)
        self.welcome_page.project_single_clicked.connect(self.enable_project_export)

        #Adds import project to file box
        self.actionProjectImport = QtWidgets.QAction(MainWindow)
        self.actionProjectImport.setObjectName("actionProjectImport")
        self.actionProjectImport.setText("Import Project")
        self.menufiles.addAction(self.actionProjectImport)
        self.actionProjectImport.setEnabled(True)        

        #Creates and adds path to home screen
        self.actionHome = QtWidgets.QAction(MainWindow)
        self.actionHome.setObjectName("actionHome")
        self.actionHome.setText("Home")

        #Adds items to help menu bar

        #Adds quick help to help menu
        self.actionQuickHelp = QtWidgets.QAction(MainWindow)
        self.actionQuickHelp.setObjectName("actionQuickHelp")
        self.actionQuickHelp.setText("Quick Help")
        self.menuhelp.addAction(self.actionQuickHelp)
        self.actionQuickHelp.setEnabled(True)

        #Adds user guide to help menu
        self.actionUserGuide = QtWidgets.QAction(MainWindow)
        self.actionUserGuide.setObjectName("actionUserGuide")
        self.actionUserGuide.setText("Open User Guide")
        self.menuhelp.addAction(self.actionUserGuide)
        self.actionUserGuide.setEnabled(True)

        #Adds project webistie to help menu
        self.actionProjectWebsite = QtWidgets.QAction(MainWindow)
        self.actionProjectWebsite.setObjectName("actionProjectWebsite")
        self.actionProjectWebsite.setText("Open Design Team Website")
        self.menuhelp.addAction(self.actionProjectWebsite)
        self.actionProjectWebsite.setEnabled(True)

        #Defines the order of sub menus in the menu bar
        self.menubar.addAction(self.actionHome)
        self.menubar.addAction(self.menuproject.menuAction())
        self.menubar.addAction(self.menufiles.menuAction())
        self.menubar.addAction(self.menuhelp.menuAction())

        #No idea what this does (at least I'm being honest)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

        #Logic (what do these buttons do)
        self.actionHome.triggered.connect(self.goHome)
        self.actionImportFile.triggered.connect(self.openImportFileMenu)
        self.actionImportRevision.triggered.connect(self.createRevisionForm)
        self.actionDefaultGraphSettings.triggered.connect(self.settings)
        self.actionCreateTemplate.triggered.connect(self.createTemplate)
        self.actionApplyTemplate.triggered.connect(self.applyTemplate)
        self.actionExport.triggered.connect(self.export) 
        self.actionProjectExport.triggered.connect(self.welcome_page.exportProject)
        self.actionProjectImport.triggered.connect(self.welcome_page.importProject)
        self.actionQuickHelp.triggered.connect(self.openQuickHelp)
        self.actionUserGuide.triggered.connect(self.openUserGuide)
        self.actionProjectWebsite.triggered.connect(self.openAbout)
        self.loadProject()

       

    def goHome(self):
        """When the user presses on the Home button, this function will take them to the Project page."""
        self.projects = []
        self.initUI()

    def openAbout(self):
        """When the user presses on the About button, this function will take them to the Senior Design Website."""
        url = QtCore.QUrl('http://sddec20-29.sd.ece.iastate.edu')
        if not QtGui.QDesktopServices.openUrl(url):
            QtGui.QMessageBox.warning(self, 'Open Url', 'Could not open Senior Design Website')

    def openUserGuide(self):
        """When the user presses on the Help button, this function will take them to our User Guide on the Senior Design Website."""
        url = QtCore.QUrl('https://sddec20-29.sd.ece.iastate.edu/GraphKey_User_Guide.pdf')
        if not QtGui.QDesktopServices.openUrl(url):
            QtGui.QMessageBox.warning(self, 'Open Url', 'Could not open User Guide')

    def openQuickHelp(self):
        """Open the quick help menu"""
        DialogHelp = QtWidgets.QDialog()
        helpbox = help.Help()
        helpbox.setupUi(DialogHelp)
        DialogHelp.setModal(True)
        DialogHelp.exec()


    def showWelcomePage(self):
        """Display the welcome page and disable certain UI elements that shouldn't be accessible while on the welcome page."""
        self.actionImportFile.setEnabled(False)
        self.actionImportRevision.setEnabled(False)
        self.actionDefaultGraphSettings.setEnabled(False)
        self.actionExport.setEnabled(False)
        self.actionCreateTemplate.setEnabled(False)
        self.actionApplyTemplate.setEnabled(False)
        MainWindow.setCentralWidget(self, self.welcome_page)

    def showGraphCreationPage(self):
        """Display the graph greation page and disable/enable certain UI elements that should/shouldn't be accessible while on the graph creation page."""
        self.actionImportFile.setEnabled(True)
        self.actionDefaultGraphSettings.setEnabled(True)
        if self.graph_creation_page.workbook_select_dropdown.count() is 0: 
            self.actionImportRevision.setEnabled(False)
        else: 
            self.actionImportRevision.setEnabled(True)
        self.actionExport.setEnabled(True)
        self.actionCreateTemplate.setEnabled(True)
        self.actionApplyTemplate.setEnabled(True)
        self.resize(1342, 944)
        center_screen = QApplication.desktop().screen().rect().center()
        self.move(center_screen - self.rect().center())
        MainWindow.setCentralWidget(self, self.graph_creation_page)

    def createProjectForm(self):
        """
        Generates the create a project popup
        """
        Dialog = create_project.CreateProject()
        Dialog.setWindowTitle("New Project")
        #prototype = create_project.CreateProject()
        Dialog.setModal(True)
        if (Dialog.exec() == QDialog.Accepted):
            newproject = Dialog.createProjectSettings()
            self.saveProject(newproject)
        

    def saveProject(self, project:project.Project):
        """Creates a project's file structure
           :param: project - The project object containing information used to save a project.
        """
        project_path_str=path_globals.projects_folder_path_str+sanitize_filepath(project.name)
        if not os.path.exists(project_path_str): 
            os.makedirs(project_path_str) 
            pickle.dump(project, open(pathlib.Path(project_path_str+"/projectconfig.pickle"), "wb")) 
            self.welcome_page.addProject(project)
            self.projects.append(project)
        else: 
            self.show_alert("Project name already exists. Failed to create Project.")

    def importedProject(self, projectName:str):
        """Adds an imported project to the welcome page project list and adds the project to the main window global variable containing a list of project objects.
           :param: projectName
        """
        imported_project = project.Project(projectName)
        self.welcome_page.addProject(imported_project)
        self.projects.append(imported_project)


    def openProject(self, row):
        """
        Opens a selected project
        """
        self.loadedProject = self.projects[row]
        self.graph_creation_page = GraphCreationWindow(path_globals.projects_folder_path_str+sanitize_filepath(self.projects[row].name), self.app)
        self.showGraphCreationPage()

    def loadProject(self):
        """
        Loads each folder, load the name specified in project pickle
        """
        if not os.path.isdir(path_globals.projects_folder_path_str):
            os.makedirs(path_globals.projects_folder_path_str) 
        for content in os.listdir(path_globals.projects_folder_path_str):
            if os.path.isdir(path_globals.projects_folder_path_str+content) and os.path.isfile(path_globals.projects_folder_path_str+content+"/projectconfig.pickle"):
                loadedproject = pickle.load(open(path_globals.projects_folder_path_str+content+"/projectconfig.pickle",'rb'))
                self.welcome_page.addProject(loadedproject)
                self.projects.append(loadedproject)

    def processDataImport(self, fileName):
        """Processes a data import (creates the appropriate file structure for a data workbook import)
           :param: fileName - The path to the file being processed for import.
        """
        try:
            newFolder = Path(fileName[0]).stem        
            os.makedirs(path_globals.projects_folder_path_str+sanitize_filepath(self.loadedProject.name)+"/"+sanitize_filepath(newFolder))
            #Creates folder for each sheet. 
            sheets = import_file.DataImport().import_data(Path(fileName[0]))
 
            #Creates a folder to store each edition in
            os.makedirs(path_globals.projects_folder_path_str+sanitize_filepath(self.loadedProject.name)+"/"+sanitize_filepath(newFolder)+"/editions")

            #Creates a folder to store each sheet folder in
            os.makedirs(path_globals.projects_folder_path_str+sanitize_filepath(self.loadedProject.name)+"/"+sanitize_filepath(newFolder)+"/sheets")

            #Creates a folder for the first edition
            os.makedirs((path_globals.projects_folder_path_str+sanitize_filepath(self.loadedProject.name)+"/"+sanitize_filepath(newFolder)+"/editions/1"))

            #Copies the excel file for safekeeping (not needed)
            copyfile(fileName[0], path_globals.projects_folder_path_str+sanitize_filepath(self.loadedProject.name)+"/"+sanitize_filepath(newFolder)+"/editions/1/1"+Path(fileName[0]).suffix)

            #Create folder to store pickled data_frames in
            os.makedirs(path_globals.projects_folder_path_str+sanitize_filepath(self.loadedProject.name)+"/"+sanitize_filepath(newFolder)+"/editions/1/1-pickles")


            for sheet in sheets:
                os.makedirs(path_globals.projects_folder_path_str+sanitize_filepath(self.loadedProject.name)+"/"+sanitize_filepath(newFolder)+"/sheets/"+sanitize_filepath(sheet))
                write_data_frame_to_pickle(sheets.get(sheet), path_globals.projects_folder_path_str+sanitize_filepath(self.loadedProject.name)+"/"+sanitize_filepath(newFolder)+"/editions/1/1-pickles/"+sanitize_filepath(sheet))
            self.graph_creation_page.updateImportDropdown(sanitize_filepath(newFolder))
            self.actionImportRevision.setEnabled(True)
            self.graph_creation_page.file_tree_view.setEnabled(True)
            self.graph_creation_page.top_left_frame.setEnabled(True)
            self.graph_creation_page.top_right_frame.setEnabled(True)
        except Exception:
            self.show_alert("Import failed. Please make sure the file type is supported and that the file has not already been imported before. If the latter is the case, use Import Revision instead.")


    def processDataRevision(self, workbook, fileName):
        """Processes a workbook revision import (creates the appropriate file structure for a data revision)
           :param: workbook - The title of the workbook that is having a revision added.
           :param: fileName - The path to the new workbook revision.
        """
        try:
            currenteditions = next(os.walk(path_globals.projects_folder_path_str+sanitize_filepath(self.loadedProject.name)+"/"+sanitize_filepath(workbook)+"/editions/"))[1]
            editionNumber = str(len(currenteditions)+1)
            #Make a directory for the new edition
            os.makedirs(path_globals.projects_folder_path_str+sanitize_filepath(self.loadedProject.name)+"/"+sanitize_filepath(workbook)+"/editions/"+editionNumber)
            #Copy file into that directory
            copyfile(fileName, path_globals.projects_folder_path_str+sanitize_filepath(self.loadedProject.name)+"/"+sanitize_filepath(workbook)+"/editions/"+editionNumber+"/"+editionNumber+Path(fileName).suffix)
            #Creates folder for each sheet. 
            sheets = import_file.DataImport().import_data(path_globals.projects_folder_path_str+sanitize_filepath(self.loadedProject.name)+"/"+sanitize_filepath(workbook)+"/editions/"+editionNumber+"/"+editionNumber+Path(fileName).suffix)
            #Create folder to store pickled data_frames in
            os.makedirs(path_globals.projects_folder_path_str+sanitize_filepath(self.loadedProject.name)+"/"+sanitize_filepath(workbook)+"/editions/"+editionNumber+"/"+editionNumber+"-pickles")
            for sheet in sheets:
                write_data_frame_to_pickle(sheets.get(sheet), path_globals.projects_folder_path_str+sanitize_filepath(self.loadedProject.name)+"/"+sanitize_filepath(workbook)+"/editions/"+editionNumber+"/"+editionNumber+"-pickles/"+sanitize_filepath(sheet))
            self.graph_creation_page.updateEditionDropdown(editionNumber)
        except Exception:
            self.show_alert("Import failed. Please make sure the file type is supported.")

    def openImportFileMenu(self):
        """
        Opens the dialog popup to import a file.
        """
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        home_dir = str(Path.home())
        fileName = QFileDialog.getOpenFileName(self,"Select File to Import", home_dir,"Excel Files (*.xlsx);;All Files (*)")
        if fileName and fileName[0]:           
            for i in range(self.graph_creation_page.workbook_select_dropdown.count()):
                if sanitize_filepath(Path(fileName[0]).stem) == self.graph_creation_page.workbook_select_dropdown.itemText(i):
                    self.show_alert("Import failed. Please make sure the file has not already been imported before. If this is the case, use Import Revision instead.")
                    return
            try:
                worker = background_worker.BackgroundWorker(self.processDataImport, fileName)
                worker.signals.started.connect(self.dataImportStarted)
                worker.signals.finished.connect(self.dataImportFinished)
                self.threadpool.start(worker)
            except Exception:
               self.show_alert("Import failed. Please make sure the file type is supported and that the file has not already been imported before. If the latter is the case, use Import Revision instead.")


    def createRevisionForm(self):
        """
        Opens a file import dialog popup to import a workbook revision.
        """
        createRevisionDialog = ImportRevision([self.graph_creation_page.workbook_select_dropdown.itemText(i) for i in range(self.graph_creation_page.workbook_select_dropdown.count())])
        createRevisionDialog.setWindowTitle("Add Workbook Edition")
        #prototype = create_project.CreateProject()
        createRevisionDialog.setModal(True)  
        if (createRevisionDialog.exec() == QDialog.Accepted):
            try:
                worker = background_worker.BackgroundWorker(self.processDataRevision, createRevisionDialog.workbook_dropdown.currentText(),createRevisionDialog.fileSelected[0])
                worker.signals.started.connect(self.dataImportStarted)
                worker.signals.finished.connect(self.dataImportFinished)
                self.threadpool.start(worker)
            except Exception:
                self.show_alert("Import failed. Please make sure the file type is supported.")


    def dataImportStarted(self):
        """
        Displays a window while specifying that a data import is occruing.
        """
        self.msg = QMessageBox()
        self.msg.setWindowTitle("Importing")
        self.msg.setText("Please wait while the file imports. Do not close this window.")
        self.msg.setWindowFlags(QtCore.Qt.CustomizeWindowHint | QtCore.Qt.WindowTitleHint | QtCore.Qt.WindowStaysOnTopHint)
        self.msg.setModal(True)
        self.app.processEvents()
        self.msg.show()
        self.app.processEvents()
    
    def dataImportFinished(self):
        """
        Closes the window specifying that a data import is ocurring and signals the graph creation page to update UI elements for selecting a workbook.
        """
        self.msg.close()
        self.graph_creation_page.workbook_selected(self.graph_creation_page.workbook_select_dropdown.currentText())

    def settings(self):
        """This function shows the Default Graph Preferences screen."""
        setting = Settings(0) 
        setting.show()  
            
    def viewTemplates(self):
        """
        Displays the template viewing window.
        """
        self.view_templates = view_templates.ViewTemplates()
        MainWindow.setCentralWidget(self, self.view_templates)
    
    def createTemplate(self):
        """
        Opens up a diaglog to create and save a template.
        """
        currentGraphPath = self.graph_creation_page.selected_pickle_file
        if currentGraphPath == None:
            self.show_alert("No graph has been selected")
            return

        title, ok = QInputDialog.getText(self, "Template", "Enter name for template:")
        if ok:
            # Things to save to template:
            #   color theme, shape theme, dimensions of png, is log scale x, is log scale y
            filename = path_globals.templates_folder_path_str + title + ".template"
            graph_data = getDataFromPickle(currentGraphPath)

            # Some graph may be box or bar plots which have no symbols
            symbols = None
            try:
                symbols = graph_data["symbol_sequence"]
            except:
                symbols = None

            template_data = {
                "color_discrete_sequence": graph_data["color_discrete_sequence"],
                "symbol_sequence": symbols,
                "width": graph_data["width_inches"],
                "height": graph_data["height_inches"],
                "log_x": graph_data["log_x"],
                "log_y": graph_data["log_y"]
            }
            createTemplatePickle(filename, template_data)

    def applyTemplate(self):
        """
        Applys a template  to the graph files for the selected sheet in the graph creation window.
        """
        graph = self.graph_creation_page
        
        folder_path = graph.selected_dataset_folder / "sheets" / graph.sheetSelected
        list_of_files = os.listdir(folder_path)
        list_of_file_paths = []
        for f in list_of_files:
            filepath = folder_path / f
            list_of_file_paths.append(filepath)
        templates = Templates(list_of_file_paths)

    def export(self):
        """This function shows the Export screen where the user can choose to export files to the local machine or Google Drive."""
        try:
            folder_path = self.graph_creation_page.selected_dataset_folder / "sheets" / self.graph_creation_page.sheetSelected 
            list_of_files = os.listdir(folder_path)
            list_of_file_paths = []
            for f in list_of_files:
                filepath = folder_path / f
                list_of_file_paths.append(filepath)
            export = ExportScreen(self.app, list_of_file_paths, self.graph_creation_page)
            export.show()
        except Exception:
            self.show_alert("No files to export")

    def show_alert(self, text):
        """Shows a basic alert message.
           :param: text - The message to show.
        """
        alert = QMessageBox()
        alert.setWindowTitle("Alert")
        alert.setText(text)
        alert.setStandardButtons(QMessageBox.Ok)
        alert.exec_()

    def enable_project_export(self):
        """
        Enables the project export menu option.
        """
        self.actionProjectExport.setEnabled(True)
