from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from .background_worker_signals import *

class BackgroundWorker(QRunnable):
    """Class for running a function with a separate thread"""
    finished = pyqtSignal()

    def __init__(self, function, *arguments, **keywordarguments):
        """Create a BackgroundWorker object.
           :param: function - The function to be run by the background worker.
           :param: *arguments - An iterable containing the arguments for the function.
           :param: **keywordarguments - A dictionary mapping parameter names as strings to their values.
        """
        super(BackgroundWorker, self).__init__()
        self.function = function
        self.args = arguments
        self.kwargs = keywordarguments
        self.signals = BackgroundWorkerSignals()

    @pyqtSlot()
    def run(self):
        """
        Initialise the run function with passed args, kwargs.
        """
        self.signals.started.emit()
        self.function(*self.args, **self.kwargs)
        self.signals.finished.emit()
