from PyQt5 import QtWidgets,QtCore,QtGui
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from PyQt5.QtGui import *
import sys, os
import pickle
from GraphKey.app.modules import path_globals

'''
This class is called from main_window.py when the "View Templates"
button is clicked. You can view the contents of a template and
remove templates.
'''
class ViewTemplates(QWidget):

    current_template = None

    def __init__(self):
        super().__init__()
        self.config_UI()
        
    def config_UI(self):
        self.setWindowTitle("Templates")
        self.resize(800, 500)

        # Template files
        self.template_model = QFileSystemModel(self)
        self.template_model.setRootPath(path_globals.templates_folder_path_str)
        self.template_model.setNameFilters(["*.template"])
        self.template_model.setNameFilterDisables(False)
        templates = QTreeView()
        templates.setModel(self.template_model)
        templates.setRootIndex(self.template_model.index(path_globals.templates_folder_path_str))
        templates.setColumnHidden(1, True)
        templates.setColumnHidden(2, True)
        templates.setColumnHidden(3, True)
        templates.setHeaderHidden(True)
        templates.clicked.connect(self.template_clicked)

        # Remove template
        remove = QPushButton("Remove Template", self)
        remove.setFixedWidth(150)
        remove.clicked.connect(self.remove)

        # Template info
        templateInfo = QLabel("Template Info:", self)
        font = QFont()
        font.setBold(True)
        templateInfo.setFont(font)
        self.pngWidth = QLabel("PNG Width: ", self)
        self.pngHeight = QLabel("PNG Height: ", self)
        self.logX = QLabel("Log Scale x-axis: ", self)
        self.logY = QLabel("Log Scale y-axis: ", self)
        self.symbols = QLabel("Symbol Sequence: ", self)
        self.colors = QLabel("Color Theme: ", self)

        vertical_layout = QVBoxLayout()
        vertical_layout.addWidget(templates)
        vertical_layout.addWidget(remove)

        self.info_layout = QVBoxLayout()
        self.info_layout.addWidget(templateInfo)
        self.info_layout.addWidget(self.pngWidth)
        self.info_layout.addWidget(self.pngHeight)
        self.info_layout.addWidget(self.logX)
        self.info_layout.addWidget(self.logY)
        self.info_layout.addWidget(self.symbols)
        self.info_layout.addWidget(self.colors)
        info_widget = QWidget()
        info_widget.setLayout(self.info_layout)

        # For showing colors
        self.color_layout = QVBoxLayout()
        color_widget = QWidget()
        color_widget.setLayout(self.color_layout)
        self.info_layout.addWidget(color_widget)

        main_layout = QHBoxLayout()
        vertical_widget = QWidget()
        vertical_widget.setLayout(vertical_layout)
        main_layout.addWidget(vertical_widget)
        main_layout.addWidget(info_widget)

        self.setLayout(main_layout)
        self.setUpdatesEnabled(True)
        self.show()

    def template_clicked(self, index):
        '''
        Retrieves the data from the template's pickle file and displays it in the UI
        '''
        self.current_template = self.template_model.fileInfo(index).absoluteFilePath()
        infile = open(self.current_template, 'rb')
        data = pickle.load(infile)
        infile.close()

        self.pngWidth.setText("Width (inches): " + str(data["width"]))
        self.pngHeight.setText("Height (inches): " + str(data["height"]))
        self.logX.setText("Log Scale x-axis: " + str(data["log_x"]))
        self.logY.setText("Log Scale y-axis: " + str(data["log_y"]))
        self.symbols.setText("Symbol Sequence: " + self.buildSymbolString(data["symbol_sequence"]))
        self.setColors(data["color_discrete_sequence"])

    def setColors(self, colors):
        '''
        First removes the color labels in the color_layout then adds the right colors
        '''

        # Remove any colors that were added
        for i in reversed(range(self.color_layout.count())):
            self.color_layout.itemAt(i).widget().setParent(None)

        for c in colors:
            cLabel = QLabel("", self)
            cLabel.setStyleSheet("background-color: " + c)
            self.color_layout.addWidget(cLabel)

    def buildSymbolString(self, symbols):
        if symbols == None:
            return "None"
        else:
            string = ""
            for s in symbols:
                string += "\n" + s
            return string 


    def remove(self):
        '''
        Removes a template file
        '''
        if self.current_template == None:
            self.show_alert("You must select a template to remove")
        else:
            alert = QMessageBox()
            alert.setText("Are you sure you want to remove this template?")
            alert.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
            result = alert.exec_()
            if result == QMessageBox.Yes:
                os.remove(self.current_template)

    def show_alert(self, text):
            alert = QMessageBox()
            alert.setText(text)
            alert.setStandardButtons(QMessageBox.Ok)
            alert.exec_()

def main():
    app = QApplication(sys.argv)
    viewTemplates = ViewTemplates()
    sys.exit(app.exec_())

if __name__ == '__main__':
    main()